@@ -1,22 +1,18 @@
 #!/bin/sh
 # default, host is empty, no cross compilation
-#host=i586-pc-msdosdjgpp
-#host=i386-pc-cygwin
-#host=i386-pc-mingw32 not tested
+# ./compile.sh [ i586-pc-msdosdjgpp | i386-pc-cygwin | i386-pc-mingw32 | powerpc-mac-darwin ]
 # Comment the version definition to not compile the library
-VER_TESTDISK=6.1
 VER_E2FSPROGS=
 VER_PROGSREISERFS=0.3.1-rc8
-VER_NTFSPROGS=1.9.4
-#VER_NTFSPROGS=1.12.0
-
-host=
-prefix=/usr/$host
-if [ "$CC" == "gcc295" ];
+#VER_NTFSPROGS=1.9.4
+VER_NTFSPROGS=1.13.1
+crosscompile_target=
+prefix=/usr/
+if [ "$CC" = "gcc295" ];
 then
   VER_NTFSPROGS=
 fi
-if [ -z $1 ];
+if [ -z "$1" ];
 then
   compiledir=.
 else
@@ -24,15 +20,20 @@ else
   if [ "$1" != "$CC" ];
   then
     VER_E2FSPROGS=1.38
-    host=$1
-    CC=$host-gcc
-#    CC="$host-gcc -mno-cygwin"
+    crosscompile_target=$1
+    TESTDISKCC=$crosscompile_target-gcc
+#    CC="$crosscompile_target-gcc -mno-cygwin"
     export CC
   fi
 fi
+prefix=/usr/$crosscompile_target
 LD=ld
 export LD
 AR=ar
+if [ "X$crosscompile_target" = "Xpowerpc-apple-darwin" ];
+then
+  AR=$crosscompile_target-ar
+fi
 export AR
 LYNX=links
 LIBEXT=$compiledir/e2fsprogs-$VER_E2FSPROGS/lib/ext2fs/libext2fs.a
@@ -66,14 +67,18 @@ then
   fi
 fi
 
-if [ -d $compiledir/e2fsprogs-$VER_E2FSPROGS ];
+if [ -e $compiledir/e2fsprogs-$VER_E2FSPROGS/configure ];
 then
   if [ ! -e $compiledir/e2fsprogs-$VER_E2FSPROGS/Makefile ];
   then
   	cd $compiledir/e2fsprogs-$VER_E2FSPROGS
-	CFLAGS="$CFLAGS -DOMIT_COM_ERR" ./configure --host=$host --prefix=$prefix
+	CC=$TESTDISKCC CFLAGS="$CFLAGS -DOMIT_COM_ERR" ./configure --host=$crosscompile_target --prefix=$prefix
 	cd $pwd_saved
   fi
+fi
+
+if [ -e $compiledir/e2fsprogs-$VER_E2FSPROGS/Makefile ];
+then
   if [ ! -e $LIBEXT ];
   then
 	cd $compiledir/e2fsprogs-$VER_E2FSPROGS
@@ -99,14 +104,18 @@ then
   fi
 fi
 
-if [ -d $compiledir/progsreiserfs-$VER_PROGSREISERFS ];
+if [ -e $compiledir/progsreiserfs-$VER_PROGSREISERFS/configure ];
 then
   if [ ! -e $compiledir/progsreiserfs-$VER_PROGSREISERFS/Makefile ];
   then
 	cd $compiledir/progsreiserfs-$VER_PROGSREISERFS
-	./configure --host=$host --prefix=$prefix --disable-nls --disable-Werror
+	./configure --host=$crosscompile_target --prefix=$prefix --disable-nls --disable-Werror
 	cd $pwd_saved
   fi
+fi
+
+if [ -e $compiledir/progsreiserfs-$VER_PROGSREISERFS/Makefile ];
+then
   if [ ! -e $LIBREISER ];
   then
 	cd $compiledir/progsreiserfs-$VER_PROGSREISERFS
@@ -132,15 +141,19 @@ then
   fi
 fi
 
-if [ -d $compiledir/ntfsprogs-$VER_NTFSPROGS ];
+if [ -e $compiledir/ntfsprogs-$VER_NTFSPROGS/configure ];
 then
   if [ ! -e $compiledir/ntfsprogs-$VER_NTFSPROGS/Makefile ];
   then
 	cd $compiledir/ntfsprogs-$VER_NTFSPROGS
 # --disable-default-device-io-ops is need for NT 4
-	./configure --host=$host --prefix=$prefix --disable-default-device-io-ops
+	CC=$TESTDISKCC ./configure --host=$crosscompile_target --prefix=$prefix --disable-default-device-io-ops
 	cd $pwd_saved
   fi
+fi
+
+if [ -e $compiledir/ntfsprogs-$VER_NTFSPROGS/Makefile ];
+then
   if [ ! -e $LIBNTFS ];
   then
 	cd $compiledir/ntfsprogs-$VER_NTFSPROGS
@@ -151,17 +164,22 @@ fi
 fi
 
 echo "Try to compile TestDisk"
+CC=$TESTDISKCC
+export CC
 
 if [ -d $compiledir ];
 then
   if [ ! -e $compiledir/Makefile ];
   then
   	cd $compiledir
-	$confdir/configure --host=$host $CONFIGUREOPT
+	$confdir/configure --host=$crosscompile_target --prefix=$prefix $CONFIGUREOPT
+	cd $pwd_saved
+  fi
+  if [ -e $compiledir/Makefile ];
+  then
+  	cd $compiledir
+	make static
 	cd $pwd_saved
   fi
-  cd $compiledir
-  make static
-  cd $pwd_saved
 fi
 
