@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -40,6 +40,7 @@
 #include "testdisk.h"
 #include "fat.h"
 #include "ntfs.h"
+#include "hfsp.h"
 #include "adv.h"
 #include "analyse.h"
 #include "intrface.h"
@@ -48,8 +49,31 @@
 #define INTER_ADV_Y	23
 #define INTER_ADV	15
 
+static int is_hfsp(const t_partition *partition);
 static int is_linux(const t_partition *partition);
 
+static int is_hfsp(const t_partition *partition)
+{
+  switch(partition->part_type_i386)
+  {
+    case P_HFSP:
+      return 1;
+  }
+  switch(partition->part_type_mac)
+  {
+    case PMAC_HFS:
+      return 1;
+  }
+  switch(partition->upart_type)
+  {
+    case UP_HFSP:
+      return 1;
+    default:
+      break;
+  }
+  return 0;
+}
+
 static int is_linux(const t_partition *partition)
 {
   switch(partition->part_type_i386)
@@ -76,6 +100,7 @@ static int is_linux(const t_partition *partition)
     case UP_RFS:
     case UP_RFS2:
     case UP_RFS3:
+    case UP_RFS4:
     case UP_XFS:
     case UP_XFS2:
     case UP_XFS3:
@@ -108,9 +133,9 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
   {
     static struct MenuItem menuAdv[]=
     {
-      {'t',"Type","Change type, this settings will not be saved on disk"},
+      {'t',"Type","Change type, this setting will not be saved on disk"},
       {'b',"Boot","Boot sector recovery"},
-      {'s',"Superblock","Locate EXT2/EXT3 backup superblock"},
+      {'s',"Superblock",NULL},
       {'q',"Quit","Return to main menu"},
       {0,NULL,NULL}
     };
@@ -153,6 +178,13 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       else if(is_linux(current_element->part))
       {
 	options="tsq";
+	menuAdv[2].desc="Locate EXT2/EXT3 backup superblock";
+	menu=1;
+      }
+      else if(is_hfsp(current_element->part))
+      {
+	options="tsq";
+	menuAdv[2].desc="Locate HFS+ backup superblock";
 	menu=1;
       }
     }
@@ -245,7 +277,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	  }
 	  else if(is_ntfs(current_element->part))
 	  {
-	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind,current_cmd);
+	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind, expert, current_cmd);
 	    rewrite=1;
 	  }
 	}
@@ -254,9 +286,16 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       case 'S':
 	if(current_element!=NULL)
 	{
-	  t_list_part *list_sb=search_superblock(disk_car,current_element->part,debug,dump_ind,1);
-	  interface_superblock(disk_car,list_sb,current_cmd);
-	  delete_list_part(list_sb);
+	  if(is_linux(current_element->part))
+	  {
+	    t_list_part *list_sb=search_superblock(disk_car,current_element->part,debug,dump_ind,1);
+	    interface_superblock(disk_car,list_sb,current_cmd);
+	    delete_list_part(list_sb);
+	  }
+	  if(is_hfsp(current_element->part))
+	  {
+	    HFSP_boot_sector(disk_car, current_element->part, debug, dump_ind, expert, current_cmd);
+	  }
 	  rewrite=1;
 	}
 	break;
@@ -264,7 +303,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       case 'T':
 	if(current_element!=NULL)
 	{
-	  change_part_type(disk_car,current_element->part);
+	  change_part_type(disk_car,current_element->part, current_cmd);
 	  rewrite=1;
 	}
 	break;
@@ -317,6 +356,9 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       {
 	aff_buffer(BUFFER_ADD,"OK\n");
       }
+      aff_buffer(BUFFER_ADD,"\n");
+      aff_buffer(BUFFER_ADD,"A valid FAT Boot sector must be present in order to access\n");
+      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
       rescan=0;
     }
     if(*current_cmd!=NULL)
@@ -433,8 +475,15 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"OK\n");
-	opt_O=1;
-	opt_over=1;
+	if(partition->upart_type==UP_FAT32)
+	{
+	  opt_O=1;
+	  opt_over=1;
+	}
+	else
+	{
+	  aff_buffer(BUFFER_ADD,"Warning: valid FAT bootsector but not a FAT32 one!");
+	}
       }
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
       if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
@@ -445,8 +494,15 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,debug,0)==0)
       {
 	aff_buffer(BUFFER_ADD,"OK\n");
-	opt_B=1;
-	opt_over=1;
+	if(partition->upart_type==UP_FAT32)
+	{
+	  opt_B=1;
+	  opt_over=1;
+	}
+	else
+	{
+	  aff_buffer(BUFFER_ADD,"Warning: valid FAT backup bootsector but not a FAT32 one!");
+	}
       }
       aff_buffer(BUFFER_ADD,"\n");
       if((memcmp(buffer_bs,buffer_backup_bs,0x3E8)==0)&&(memcmp(&buffer_bs[0x3F0],&buffer_backup_bs[0x3F0],0x600-0x3F0))==0)
@@ -464,6 +520,9 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	if(memcmp(&buffer_bs[2*disk_car->sector_size],&buffer_backup_bs[2*disk_car->sector_size],DEFAULT_SECTOR_SIZE)!=0)
 	  aff_buffer(BUFFER_ADD,"Third sectors (Second part of boot code) are not identical.\n");
       }
+      aff_buffer(BUFFER_ADD,"\n");
+      aff_buffer(BUFFER_ADD,"A valid FAT Boot sector must be present in order to access\n");
+      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
       if(opt_over!=0)
       {
 	if(opt_B!=0 && opt_O!=0)
@@ -569,7 +628,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
   }
 }
 
-int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, char **current_cmd)
+int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char buffer_bs[DEFAULT_SECTOR_SIZE];
   unsigned char buffer_backup_bs[DEFAULT_SECTOR_SIZE];
@@ -583,6 +642,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
     { 'O', "Org. BS","Copy boot sector over backup sector"},
     { 'B', "Backup BS","Copy backup boot sector over boot sector"},
     { 'R', "Rebuild BS","Rebuild boot sector"},
+    { 'M', "Repair MFT","Check MFT"},
     { 'D', "Dump","Dump boot sector and backup boot sector"},
     { 0, NULL, NULL }
   };
@@ -596,7 +656,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       int opt_over=0;
       int opt_B=0;
       int opt_O=0;
-      options="DR";
+      options="DRM";
       aff_copy(stdscr);
       wmove(stdscr,4,0);
       wdoprintf(stdscr,"%s",disk_car->description(disk_car));
@@ -649,6 +709,9 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	dump_2ntfs_rapport((const struct ntfs_boot_sector *)buffer_bs, (const struct ntfs_boot_sector *)buffer_backup_bs);
 	aff_buffer(BUFFER_ADD,"Sectors are not identical.\n");
       }
+      aff_buffer(BUFFER_ADD,"\n");
+      aff_buffer(BUFFER_ADD,"A valid NTFS Boot sector must be present in order to access\n");
+      aff_buffer(BUFFER_ADD,"any data; even if the partition is not bootable.\n");
       if(opt_over!=0)
       {
 	if(opt_B!=0 && opt_O!=0)
@@ -717,8 +780,11 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	  rescan=1;
 	}
 	break;
+      case 'M':
+        repair_MFT(disk_car, partition, debug);
+	break;
       case 'R': /* R : rebuild boot sector */
-	rebuild_NTFS_BS(disk_car,partition,debug,dump_ind,1,current_cmd);
+	rebuild_NTFS_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
 	rescan=1;
 	break;
       case 'D':
@@ -744,3 +810,166 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
   }
 }
 
+int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
+{
+  unsigned char buffer_bs[0x400];
+  unsigned char buffer_backup_bs[0x400];
+  const char *options="";
+  int rescan=1;
+  struct MenuItem menu_hfsp[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Quit","Return to Advanced menu"},
+    { 'O', "Org. BS","Copy superblock over backup sector"},
+    { 'B', "Backup BS","Copy backup superblock over superblock"},
+    { 'D', "Dump","Dump superblock and backup superblock"},
+    { 0, NULL, NULL }
+  };
+
+  while(1)
+  {
+    int command;
+    aff_buffer(BUFFER_RESET,"Q");
+    if(rescan==1)
+    {
+      int opt_over=0;
+      int opt_B=0;
+      int opt_O=0;
+      options="D";
+      aff_copy(stdscr);
+      wmove(stdscr,4,0);
+      wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+      mvwaddstr(stdscr,5,0,msg_PART_HEADER_LONG);
+      wmove(stdscr,6,0);
+      aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
+      ecrit_rapport("\nHFSP_boot_sector\n");
+      aff_part_rapport(disk_car,partition);
+      aff_buffer(BUFFER_ADD,"Superblock\n");
+      if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+0x400)!=0)
+      {
+	aff_buffer(BUFFER_ADD,"HFSP_boot_sector: Can't read superblock.\n");
+	memset(&buffer_bs,0,sizeof(buffer_bs));
+      }
+      if(test_HFSP(disk_car,(struct hfsp_vh*)buffer_bs,partition,debug,0)==0)
+      {
+	aff_buffer(BUFFER_ADD,"OK\n");
+	opt_O=1;
+	opt_over=1;
+      }
+      else
+      {
+	aff_buffer(BUFFER_ADD,"Bad\n");
+      }
+      aff_buffer(BUFFER_ADD,"\nBackup superblock\n");
+      if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+partition->part_size-0x400)!=0)
+      {
+	aff_buffer(BUFFER_ADD,"HFSP_boot_sector: Can't read backup superblock.\n");
+	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
+      }
+      if(test_HFSP(disk_car,(struct hfsp_vh*)buffer_backup_bs,partition,debug,0)==0)
+      {
+	aff_buffer(BUFFER_ADD,"OK\n");
+	opt_B=1;
+	opt_over=1;
+      }
+      else
+      {
+	aff_buffer(BUFFER_ADD,"Bad\n");
+      }
+      aff_buffer(BUFFER_ADD,"\n");
+      if(memcmp(buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE)==0)
+      {
+	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
+	opt_over=0;
+      }
+      else
+      {
+	aff_buffer(BUFFER_ADD,"Sectors are not identical.\n");
+      }
+      if(opt_over!=0)
+      {
+	if(opt_B!=0 && opt_O!=0)
+	  options="DOB";
+	else if(opt_B!=0)
+	  options="DB";
+	else if(opt_O!=0)
+	  options="DO";
+      }
+      rescan=0;
+    }
+    if(*current_cmd!=NULL)
+    {
+      command=0;
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"dump",4)==0)
+      {
+	(*current_cmd)+=4;
+	command='D';
+      }
+      else if(strncmp(*current_cmd,"originalhfsp",11)==0)
+      {
+	(*current_cmd)+=11;
+	if(strchr(options,'O')!=NULL)
+	    command='O';
+      }
+      else if(strncmp(*current_cmd,"backuphfsp",9)==0)
+      {
+	(*current_cmd)+=9;
+	if(strchr(options,'B')!=NULL)
+	    command='B';
+      }
+      screen_buffer_to_log();
+    }
+    else
+      command=screen_buffer_display(stdscr,options,menu_hfsp);
+    switch(command)
+    {
+      case 0:
+	return 0;
+      case 'O': /* O : copy original superblock over backup boot */
+	if(ask_confirmation("Copy original HFSP superblock over backup boot, confirm ? (Y/N)")!=0)
+	{
+	  ecrit_rapport("copy original superblock over backup boot\n");
+	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+partition->part_size-0x400)!=0)
+	  {
+	    display_message("Write error: Can't overwrite HFSP backup superblock\n");
+	  }
+	  rescan=1;
+	}
+	break;
+      case 'B': /* B : copy backup superblock over main superblock */
+	if(ask_confirmation("Copy backup HFSP superblock over main superblock, confirm ? (Y/N)")!=0)
+	{
+	  ecrit_rapport("copy backup superblock over main superblock\n");
+	  if(disk_car->write(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+0x400)!=0)
+	  {
+	    display_message("Write error: Can't overwrite HFSP main superblock\n");
+	  }
+	  rescan=1;
+	}
+	break;
+      case 'D':
+	{
+	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
+	  keypad(window, TRUE); /* Need it to get arrow key */
+	  aff_copy(window);
+	  wmove(window,4,0);
+	  wdoprintf(window,"%s",disk_car->description(disk_car));
+	  wmove(window,5,0);
+	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
+	  ecrit_rapport("Superblock                        Backup superblock\n");
+	  mvwaddstr(window,6,0, "Superblock                        Backup superblock");
+	  dump2(window,buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE);
+	  delwin(window);
+	  (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	  touchwin(stdscr);
+#endif
+	}
+	break;
+    }
+  }
+}
+
