@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@
 #include "cramfs.h"
 #include "ext2.h"
 #include "fat.h"
+#include "fatx.h"
 #include "hfs.h"
 #include "hfsp.h"
 #include "jfs_superblock.h"
@@ -88,7 +89,7 @@ int search_HFS_backup(t_param_disk *disk_car,t_partition *partition, const int d
 
 int search_FAT_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
 {
-  unsigned char buffer[3*disk_car->sector_size];
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
   if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
     return -1;
   /* FAT32 recovery using backup sector */
@@ -117,6 +118,7 @@ int search_type_0(t_param_disk *disk_car,t_partition *partition, const int debug
   if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind,0)==0) return 1;
   if(recover_netware(disk_car,(const struct disk_netware *)&buffer,partition)==0) return 1;
   if(recover_xfs(disk_car,(const struct xfs_sb*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_FATX(disk_car,(const struct disk_fatx*)&buffer,partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
@@ -183,7 +185,8 @@ int search_type_64(t_param_disk *disk_car,t_partition *partition,const int debug
 
 int search_type_128(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+  unsigned char buffer[16*DEFAULT_SECTOR_SIZE];
+  /* Reiserfs4 need to read the master superblock and the format40 superblock => 2*4096 */
   if(debug>2)
   {
     ecrit_rapport("search_type_128 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
@@ -199,13 +202,13 @@ int search_type_128(t_param_disk *disk_car,t_partition *partition,const int debu
 
 t_list_part *search_superblock(t_param_disk *disk_car, const t_partition *partition, const int debug, const int dump_ind, const int interface)
 {
-  unsigned char buffer[2*0x200];
+  unsigned char *buffer=MALLOC(2*0x200);
   uint64_t hd_offset;
   int nbr_sb=0;
   t_list_part *list_part=NULL;
   int ind_stop=0;
   unsigned long int old_percent=0;
-  struct ext2_super_block *sb=(struct ext2_super_block *)&buffer;
+  struct ext2_super_block *sb=(struct ext2_super_block *)buffer;
   t_partition *new_partition=partition_new();
   ecrit_rapport("search_superblock\n");
   if(interface>0)
@@ -253,7 +256,7 @@ t_list_part *search_superblock(t_param_disk *disk_car, const t_partition *partit
       hd_offset%(5*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2))==0 ||
       hd_offset%(7*(EXT2_MIN_BLOCK_SIZE<<2)*8*(EXT2_MIN_BLOCK_SIZE<<2))==0)
     {
-      if(disk_car->read(disk_car,1024, &buffer, partition->part_offset+hd_offset)==0)
+      if(disk_car->read(disk_car,1024, buffer, partition->part_offset+hd_offset)==0)
       {
 	if(le16(sb->s_magic)==EXT2_SUPER_MAGIC)
 	{
@@ -281,6 +284,7 @@ t_list_part *search_superblock(t_param_disk *disk_car, const t_partition *partit
     }
   }
   FREE(new_partition);
+  FREE(buffer);
   return list_part;
 }
 
