@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -33,7 +33,7 @@
 #include "fnctdsk.h"
 #include "chgtype.h"
 
-int change_part_type(const t_param_disk *disk_car,t_partition *partition)
+int change_part_type(const t_param_disk *disk_car,t_partition *partition, char **current_cmd)
 {
   char response[100];
   int nbr=0;
@@ -71,13 +71,29 @@ int change_part_type(const t_param_disk *disk_car,t_partition *partition)
   aff_copy(stdscr);
   wmove(stdscr,4,0);
   aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
-  screen_buffer_display(stdscr,"",menuType);
+  if(*current_cmd==NULL)
+    screen_buffer_display(stdscr,"",menuType);
   wmove(stdscr,23,0);
   wdoprintf(stdscr,"New partition type [current %02x] ? ",disk_car->arch->get_part_type(partition));
-  if (get_string(response, sizeof(response), NULL) > 0) {
-    int tmp_val = strtol(response, NULL, 16);
-    disk_car->arch->set_part_type(partition,tmp_val);
+  if(*current_cmd!=NULL)
+  {
+    while(*current_cmd[0]==',')
+      (*current_cmd)++;
+    {
+      int tmp_val= strtol(*current_cmd, NULL, 16);
+      while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
+	(*current_cmd)++;
+      disk_car->arch->set_part_type(partition,tmp_val);
+    }
   }
+  else
+  {
+    if (get_string(response, sizeof(response), NULL) > 0) {
+      int tmp_val = strtol(response, NULL, 16);
+      disk_car->arch->set_part_type(partition,tmp_val);
+    }
+  }
+  aff_part_rapport(disk_car,partition);
   return 0;
 }
 
