@@ -14,12 +14,12 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
 void change_geometry(t_param_disk *disk_car, char **current_cmd);
-int change_part_type(const t_param_disk *disk_car,t_partition *partition);
+int change_part_type(const t_param_disk *disk_car,t_partition *partition, char **current_cmd);
 
