@@ -2,7 +2,7 @@
 
     File: common.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -36,6 +36,9 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #include "intrf.h"
 
 static unsigned int up2power_aux(const unsigned int number);
@@ -120,7 +123,9 @@ int strncasecmp(const char * s1, const char * s2, size_t len)
 
 unsigned int up2power(const unsigned int number)
 {
-/*  ecrit_rapport("up2power(%u)=>%u\n",number, (1<<up2power_aux(number-1))); */
+  /* ecrit_rapport("up2power(%u)=>%u\n",number, (1<<up2power_aux(number-1))); */
+  if(number==0)
+    return 1;
   return (1<<up2power_aux(number-1));
 }
 
@@ -171,3 +176,38 @@ void set_part_name(t_partition *partition,const char *src,const int max_size)
   partition->name[i--]='\0';
 }
 
+FILE *create_file(const char *filename)
+{
+  FILE *f_out;
+  f_out=fopen(filename,"wb");
+  if(!f_out)
+  {
+    /* create all sub-directories */
+    char *pos;
+    char *path=strdup(filename);
+    if(path==NULL)
+      return f_out;
+    pos=path+1;
+    do
+    {
+      strcpy(path,filename);
+      pos=strchr(pos+1,'/');
+      if(pos!=NULL)
+      {
+	*pos='\0';
+#ifdef HAVE_MKDIR
+#ifdef __MINGW32__
+	mkdir(path);
+#else
+	mkdir(path, 0775);
+#endif
+#else
+#error You need a mkdir function!
+#endif
+      }
+    } while(pos!=NULL);
+    FREE(path);
+    f_out=fopen(filename,"wb");
+  }
+  return f_out;
+}
