@@ -2,7 +2,7 @@
 
     File: common.h
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #define DEFAULT_SECTOR_SIZE     0x200
@@ -97,7 +97,17 @@
 #define PMAC_DRIVER	10
 #define PMAC_MFS	11
 #define PMAC_PRODOS	12
-enum upart_type { UP_BEOS, UP_CRAMFS, UP_EXT2, UP_EXT3, UP_EXTENDED, UP_FAT12, UP_FAT16, UP_FAT32, UP_FREEBSD,  UP_HFS, UP_HFSP, UP_HPFS, UP_JFS, UP_LINSWAP, UP_LINSWAP2, UP_LVM, UP_LVM2, UP_NETWARE, UP_NTFS, UP_OPENBSD, UP_OS2MB, UP_RAID, UP_RFS, UP_RFS2, UP_RFS3, UP_SUN, UP_SYSV4, UP_UFS, UP_UFS2, UP_UNK, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4};
+
+#define PXBOX_UNK	0
+#define PXBOX_FATX	1
+
+#define TESTDISK_O_RDONLY     00
+#define TESTDISK_O_RDWR	      02
+#define TESTDISK_O_DIRECT 040000
+#define TESTDISK_O_READAHEAD_8K 04
+#define TESTDISK_O_READAHEAD_32K 010
+
+enum upart_type { UP_BEOS, UP_CRAMFS, UP_EXT2, UP_EXT3, UP_EXTENDED, UP_FAT12, UP_FAT16, UP_FAT32, UP_FATX, UP_FREEBSD,  UP_HFS, UP_HFSP, UP_HPFS, UP_JFS, UP_LINSWAP, UP_LINSWAP2, UP_LVM, UP_LVM2, UP_NETWARE, UP_NTFS, UP_OPENBSD, UP_OS2MB, UP_RAID, UP_RFS, UP_RFS2, UP_RFS3, UP_RFS4, UP_SUN, UP_SYSV4, UP_UFS, UP_UFS2, UP_UNK, UP_XFS, UP_XFS2, UP_XFS3, UP_XFS4};
 typedef enum upart_type upart_type_t;
 enum status_type { STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG, STATUS_EXT, STATUS_EXT_IN_EXT};
 typedef enum status_type status_type_t;
@@ -144,6 +154,7 @@ struct systypes {
 struct arch_fnct_struct
 {
   const char *part_name;
+  const char *part_name_option;
   const char *msg_part_type;
   t_list_part *(*read_part)(t_param_disk *disk_car, const int debug,const int saveheader);
   int (*write_part)(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug, const int align);
@@ -151,7 +162,7 @@ struct arch_fnct_struct
   int (*get_geometry_from_mbr)(const unsigned char *buffer, const int debug, t_CHS *geometry);
   int (*check_part)(t_param_disk *disk_car,const int debug,t_partition *partition, const int saveheader);
   int (*write_MBR_code)(t_param_disk *disk_car);
-  t_list_part *(*add_partition)(t_param_disk *disk_car,t_list_part *list_part, const int debug);
+  t_list_part *(*add_partition)(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
   void (*set_prev_status)(const t_param_disk *disk_car, t_partition *partition);
   void (*set_next_status)(const t_param_disk *disk_car, t_partition *partition);
   int (*test_structure)(t_list_part *list_part);
@@ -175,12 +186,15 @@ struct param_disk_struct
   unsigned int sector_size;
   const char *device;
   char description_txt[DISKDESCRIPTION_MAX];
+  char description_short_txt[DISKDESCRIPTION_MAX];
   const char *(*description)(t_param_disk *disk_car);
+  const char *(*description_short)(t_param_disk *disk_car);
   int (*read)(t_param_disk *disk_car,const unsigned int nbr_sector, void *nom_buffer, const uint64_t offset);
   int (*write)(t_param_disk *disk_car,const unsigned int nbr_sector, const void *nom_buffer, const uint64_t offset);
   int (*clean)(t_param_disk *disk_car);
   const t_arch_fnct *arch;
   void *data;
+  uint64_t disk_real_size;
 };
 
 struct partition_struct
@@ -192,6 +206,7 @@ struct partition_struct
   unsigned int part_type_i386;
   unsigned int part_type_sun;
   unsigned int part_type_mac;
+  unsigned int part_type_xbox;
   upart_type_t upart_type;
   status_type_t status;
   unsigned int order;
@@ -225,14 +240,17 @@ void FREE(void *ptr);
 char read_key(void);
 char test_key(void);
 unsigned int up2power(const unsigned int number);
-void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cylinder, const t_arch_fnct *arch, const int debug);
-t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct *arch, const int mode_direct);
-t_param_disk *file_test_availability(const char *device, const int debug, const t_arch_fnct *arch, const int use_direct);
+void hd_update_geometry(t_param_disk *disk_car, const int allow_partial_last_cylinder, const int debug);
+void hd_update_all_geometry(const t_list_disk * list_disk, const int allow_partial_last_cylinder, const int debug);
+t_list_disk *hd_parse(t_list_disk *list_disk, const int debug, const t_arch_fnct *arch, const int testdisk_mode);
+t_param_disk *file_test_availability(const char *device, const int debug, const t_arch_fnct *arch, const int testdisk_mode);
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+t_param_disk *file_test_availability_win32(const char *device, const int debug, const t_arch_fnct *arch, const int testdisk_mode);
+#endif
 void my_sort(void *base, size_t nmemb, size_t size, int (*compar)(const void *, const void *));
 int check_volume_name(const char *name,const unsigned int max_size);
 void set_part_name(t_partition *partition,const char *src,const int max_size);
 int rebuild_FAT_BS(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind,const int interface, const unsigned int expert, char**current_cmd);
-int rebuild_NTFS_BS(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind,const int interface, char**current_cmd);
 #ifndef BSD_MAXPARTITIONS
 #define	BSD_MAXPARTITIONS	8
 #endif
@@ -240,7 +258,7 @@ int rebuild_NTFS_BS(t_param_disk *disk_car,t_partition *partition, const int deb
 #define	OPENBSD_MAXPARTITIONS	16
 #endif
 
-#if defined(__i386__) || defined(__alpha__)
+#ifdef TESTDISK_LSB
 #define le16(x)  (x)             /* x as little endian */
 #define be16(x)  ((((x)&0xff00)>>8)                      | \
     (((x)&0x00ff)<<8))
@@ -288,4 +306,8 @@ int snprintf(char *str, size_t size, const char *format, ...);
 #ifndef HAVE_VSNPRINTF
 int vsnprintf(char *str, size_t size, const char *format, va_list ap);
 #endif
+#ifndef HAVE_STRNCASECMP
+int strncasecmp(const char * s1, const char * s2, size_t len);
+#endif
+FILE *create_file(const char *filename);
 
