@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,9 @@
 
 static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time, char **current_cmd);
 static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time, char**current_cmd);
+#ifdef EXPERIMENTAL_COPY
+static int copy_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode);
+#endif
 
 static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
@@ -204,8 +207,36 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
   aff_copy(dir_data->window);
   wmove(dir_data->window,4,0);
   aff_part(dir_data->window,AFF_PART_ORDER,disk_car,partition);
-  mvwaddstr(dir_data->window,5,0,(first_time!=0? "Use right arrow to change directory, q to quit":
-	"Use left arrow to go back, right arrow to change directory, q to quit"));
+  mvwaddstr(dir_data->window,5,0,"Use ");
+  if(first_time==0)
+  {
+    if(has_colors())
+      wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
+    waddstr(dir_data->window, "Left");
+    if(has_colors())
+      wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+    waddstr(dir_data->window," arrow to go back, ");
+  }
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
+  waddstr(dir_data->window,"Right");
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+  waddstr(dir_data->window," arrow to change directory, ");
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
+#ifdef EXPERIMENTAL_COPY
+  waddstr(dir_data->window,"c");
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+  waddstr(dir_data->window," to copy, ");
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(0));
+#endif
+  waddstr(dir_data->window,"q");
+  if(has_colors())
+    wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+  waddstr(dir_data->window," to quit");
   wmove(dir_data->window,6,0);
   wdoprintf(dir_data->window,"Directory %s\n",dir_data->current_directory);
   do
@@ -257,6 +288,8 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
       car='q';
     else
       car=wgetch(dir_data->window);
+    wmove(dir_data->window,7,0);
+    wclrtoeol(dir_data->window);
     switch(car)
     {
       case key_ESC:
@@ -339,6 +372,45 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
 	      offset++;
 	  }
 	  break;
+#ifdef EXPERIMENTAL_COPY
+	case 'c':
+	  {
+	    unsigned int current_directory_namelength=strlen(dir_data->current_directory);
+	    if(current_directory_namelength+1+strlen(pos->name)<sizeof(dir_data->current_directory)-1)
+	    {
+	      wmove(dir_data->window,7,0);
+	      wclrtoeol(dir_data->window);
+	      if(has_colors())
+		wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(1));
+	      wdoprintf(dir_data->window,"Copying, please wait...");
+	      if(has_colors())
+		wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+	      wrefresh(dir_data->window);
+	      if(strcmp(dir_data->current_directory,"/"))
+		strcat(dir_data->current_directory,"/");
+	      strcat(dir_data->current_directory,pos->name);
+	      if(LINUX_S_ISDIR(pos->filestat.st_mode)!=0)
+	      {
+		if(strcmp(pos->name,"..")!=0)
+		  copy_dir(disk_car, partition, dir_data, (unsigned long int)pos->filestat.st_ino);
+	      }
+	      else if(LINUX_S_ISREG(pos->filestat.st_mode)!=0)
+	      {
+		if(dir_data->copy_file!=NULL)
+		  dir_data->copy_file(disk_car, partition, dir_data, (unsigned long int)pos->filestat.st_ino);
+	      }
+	      dir_data->current_directory[current_directory_namelength]='\0';
+	      wmove(dir_data->window,7,0);
+	      wclrtoeol(dir_data->window);
+	      if(has_colors())
+		wbkgdset(dir_data->window,' ' | A_BOLD | COLOR_PAIR(2));
+	      wdoprintf(dir_data->window,"Copy done!");
+	      if(has_colors())
+		wbkgdset(dir_data->window,' ' | COLOR_PAIR(0));
+	    }
+	  }
+	  break;
+#endif
       }
     }
   } while(quit==0);
@@ -389,3 +461,40 @@ static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partitio
   delete_list_file(dir_list);
   return new_inode;
 }
+
+#ifdef EXPERIMENTAL_COPY
+static int copy_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode)
+{
+  t_file_data *dir_list;
+  unsigned int current_directory_namelength=strlen(dir_data->current_directory);
+  t_file_data *current_file;
+  if(dir_data->get_dir==NULL || dir_data->copy_file==NULL)
+    return 1;
+  ecrit_rapport("copy_dir %s\n",dir_data->current_directory);
+  dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
+//  dir_aff_log(disk_car, partition, dir_data, dir_list);
+  for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
+  {
+    if(strlen(dir_data->current_directory)+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
+    {
+      if(strcmp(dir_data->current_directory,"/"))
+	strcat(dir_data->current_directory,"/");
+      strcat(dir_data->current_directory,current_file->name);
+      if(LINUX_S_ISDIR(current_file->filestat.st_mode)!=0)
+      {
+	if((unsigned long int)current_file->filestat.st_ino!=inode &&
+	    strcmp(current_file->name,"..")!=0 && strcmp(current_file->name,".")!=0)
+	  copy_dir(disk_car, partition, dir_data, (unsigned long int)current_file->filestat.st_ino);
+      }
+      else if(LINUX_S_ISREG(current_file->filestat.st_mode)!=0)
+      {
+//	ecrit_rapport("copy_file %s\n",dir_data->current_directory);
+	dir_data->copy_file(disk_car, partition, dir_data, (unsigned long int)current_file->filestat.st_ino);
+      }
+      dir_data->current_directory[current_directory_namelength]='\0';
+    }
+  }
+  delete_list_file(dir_list);
+  return 0;
+}
+#endif
