@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,7 @@ void interface_editor(t_param_disk *disk_car)
 {
   int done = FALSE;
   uint64_t hd_offset=0;
+  unsigned char *buffer=MALLOC(disk_car->sector_size);
   ecrit_rapport("%s\n",disk_car->description(disk_car));
   while (done==FALSE)
   {
@@ -63,7 +64,6 @@ void interface_editor(t_param_disk *disk_car)
       case 'd':
       case 'D':
 	{
-	  unsigned char buffer[disk_car->sector_size];
 	  int menu_pos=KEY_DOWN;
 	  while(done==0)
 	  {
@@ -71,9 +71,10 @@ void interface_editor(t_param_disk *disk_car)
 	    wclrtoeol(stdscr);
 	    wdoprintf(stdscr,"%lu ", (unsigned long)(hd_offset/disk_car->sector_size));
 	    aff_LBA2CHS(disk_car,hd_offset/disk_car->sector_size);
-	    if(disk_car->read(disk_car,sizeof(buffer), buffer, hd_offset))
+	    if(disk_car->read(disk_car,disk_car->sector_size, buffer, hd_offset))
+	    {
 	      wdoprintf(stdscr,msg_PART_RD_ERR);
-	    else
+	    }
 	    {
 	      menu_pos=dump_editor(buffer,disk_car->sector_size,menu_pos);
 	      switch(menu_pos)
@@ -106,6 +107,7 @@ void interface_editor(t_param_disk *disk_car)
 	break;
     }
   }
+  FREE(buffer);
 }
 
 static void interface_editor_position(const t_param_disk *disk_car,uint64_t *lba)
