@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -83,6 +83,7 @@ static struct struct_io_manager my_struct_manager = {
 	set_option: NULL
 #endif
 };
+static t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
 
 io_channel *shared_ioch=NULL;
 io_manager my_io_manager = &my_struct_manager;
@@ -186,64 +187,6 @@ static errcode_t my_flush(io_channel channel)
   return 0;
 }
 
-static int list_dir_proc(struct ext2_dir_entry *dirent,
-			 int	offset,
-			 int	blocksize,
-			 char	*buf,
-			 void	*private)
-{
-  struct ext2_inode	inode;
-  ext2_ino_t		ino;
-  unsigned int		thislen;
-  struct list_dir_struct *ls = (struct list_dir_struct *) private;
-  t_file_data *new_file=MALLOC(sizeof(*new_file));
-  new_file->prev=ls->current_file;
-  new_file->next=NULL;
-
-  thislen = ((dirent->name_len & 0xFF) < EXT2_NAME_LEN) ?
-    (dirent->name_len & 0xFF) : EXT2_NAME_LEN;
-  if(thislen>DIR_NAME_LEN)
-    thislen=DIR_NAME_LEN;
-  memcpy(new_file->name, dirent->name, thislen);
-  new_file->name[thislen] = '\0';
-  ino = dirent->inode;
-/*  ecrit_rapport("list_dir_proc inode %u\n",(unsigned)ino); */
-  if (ino) {
-    errcode_t retval;
-    if ((retval=ext2fs_read_inode(ls->current_fs,ino, &inode))!=0)
-    {
-      ecrit_rapport("ext2fs_read_inode(ino=%u) failed with error %ld.\n",(unsigned)ino, (long)retval);
-      memset(&inode, 0, sizeof(struct ext2_inode));
-    }
-  } else {
-    memset(&inode, 0, sizeof(struct ext2_inode));
-  }
-/*  ecrit_rapport("inode=%u uid=%u gid=%u\n",ino,inode.i_uid,inode.i_gid); */
-  new_file->filestat.st_dev=0;
-  new_file->filestat.st_ino=ino;
-  new_file->filestat.st_mode=inode.i_mode;
-  new_file->filestat.st_nlink=inode.i_links_count;
-  new_file->filestat.st_uid=inode.i_uid;
-  new_file->filestat.st_gid=inode.i_gid;
-  new_file->filestat.st_rdev=0;
-  new_file->filestat.st_size=LINUX_S_ISDIR(inode.i_mode)?inode.i_size:
-    inode.i_size| ((uint64_t)inode.i_size_high << 32);
-  new_file->filestat.st_blksize=blocksize;
-#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-  new_file->filestat.st_blocks=inode.i_blocks;
-#endif
-  new_file->filestat.st_atime=inode.i_atime;
-  new_file->filestat.st_mtime=inode.i_mtime;
-  new_file->filestat.st_ctime=inode.i_ctime;
-  if(ls->current_file!=NULL)
-    ls->current_file->next=new_file;
-  else
-    ls->dir_list=new_file;
-  ls->current_file=new_file;
-  return 0;
-}
-
-#ifdef TEST_PROC2
 static int list_dir_proc2(ext2_ino_t dir,
 			 int    entry,
 			 struct ext2_dir_entry *dirent,
@@ -305,17 +248,14 @@ static int list_dir_proc2(ext2_ino_t dir,
   ls->current_file=new_file;
   return 0;
 }
-#endif
 
-t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
-t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
+static t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
 {
   errcode_t       retval;
   struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
   ls->dir_list=NULL;
   ls->current_file=NULL;
-//  ext2fs_dir_iterate2(ls->current_fs, cluster, ls->flags, 0, list_dir_proc2, ls);
-  if((retval=ext2fs_dir_iterate(ls->current_fs, cluster, ls->flags, 0, list_dir_proc, ls))!=0)
+  if((retval=ext2fs_dir_iterate2(ls->current_fs, cluster, ls->flags, 0, list_dir_proc2, ls))!=0)
   {
     ecrit_rapport("ext2fs_dir_iterate failed with error %ld.\n",(long)retval);
   }
@@ -356,6 +296,7 @@ int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partit
     dir_data.debug=debug;
     dir_data.private_dir_data=&ls;
     dir_data.get_dir=ext2_dir;
+    dir_data.copy_file=NULL;
     strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
     dir_partition(disk_car,partition,&dir_data,EXT2_ROOT_INO,current_cmd);
   }
@@ -383,7 +324,10 @@ int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partit
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with e2fsprogs library");
-  screen_buffer_display(window,"",NULL);
+  if(*current_cmd!=NULL)
+    screen_buffer_to_log();
+  else
+    screen_buffer_display(window,"",NULL);
   return 0;
 }
 
