@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -855,8 +855,8 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
   fat_header->sector_size[1]=disk_car->sector_size >>8;
   fat_header->fats=fats;
   fat_header->media=0xF8;
-  fat_header->secs_track=disk_car->CHS.sector;
-  fat_header->heads=disk_car->CHS.head+1;
+  fat_header->secs_track=le16(disk_car->CHS.sector);
+  fat_header->heads=le16(disk_car->CHS.head+1);
   fat_header->marker=le16(0xAA55);
   if(!((fat_header->ignored[0]==0xeb&&fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9))
   {
@@ -870,9 +870,9 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
       memcmp(fat_header->system_id,"MSWIN4.0",8) &&
       memcmp(fat_header->system_id,"MSWIN4.1",8))
     memcpy(fat_header->system_id,"MSWIN4.1",8);
-  
+  /* FIXME, need to know where the extended or logical partition start */
   if(partition->status==STATUS_LOG)
-    fat_header->hidden=le32(63);
+    fat_header->hidden=le32(disk_car->CHS.sector);
   else
     fat_header->hidden=le32((partition->part_offset/disk_car->sector_size));
   fat_header->cluster_size=cluster_size;
@@ -1087,7 +1087,21 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	case 'D':
 	  if(strchr(options,'D')!=NULL)
 	  {
-	    dump2(stdscr,newboot,orgboot, (unsigned int)(upart_type==UP_FAT32?3*disk_car->sector_size:DEFAULT_SECTOR_SIZE));
+	    WINDOW *window=newwin(0,0,0,0);	/* full screen */
+	    keypad(window, TRUE); /* Need it to get arrow key */
+	    aff_copy(window);
+	    wmove(window,4,0);
+	    wdoprintf(window,"%s",disk_car->description(disk_car));
+	    wmove(window,5,0);
+	    aff_part(window,AFF_PART_ORDER,disk_car,partition);
+	    ecrit_rapport("     Rebuild Boot sector           Boot sector\n");
+	    mvwaddstr(window,6,0, "     Rebuild Boot sector           Boot sector");
+	    dump2(window,newboot,orgboot, (unsigned int)(upart_type==UP_FAT32?3*disk_car->sector_size:DEFAULT_SECTOR_SIZE));
+	    delwin(window);
+	    (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	    touchwin(stdscr);
+#endif
 	  }
 	  break;
 	case 'l':
@@ -1130,7 +1144,7 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 
 static int calcul_cluster_size(const upart_type_t upart_type, const unsigned long int data_size, const unsigned int fat_length, const unsigned int sector_size)
 {
-/* ecrit_rapport("calcul_cluster_size data_size=%lu, fat_length=%u, sector_size=%u\n",data_size,fat_length,sector_size); */
+  /* ecrit_rapport("calcul_cluster_size data_size=%lu, fat_length=%u, sector_size=%u\n",data_size,fat_length,sector_size); */
   if(fat_length==0)
     return 0;
   switch(upart_type)
@@ -1149,7 +1163,7 @@ static int calcul_cluster_size(const upart_type_t upart_type, const unsigned lon
 
 static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_fat12, const int p_fat16, const int p_fat32,unsigned long int*fat_offset, const unsigned int sector_size)
 {
-  /* TODO: Handle data_size for I/O optimization, handle limited size of info_offset */
+  /* TODO: handle limited size of info_offset */
   t_info_offset info_offset[sector_size];
   unsigned int nbr_offset=0;
   int have_fat_signature=0;
@@ -1506,6 +1520,27 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
   t_info_offset info_offset[0x200];
   upart_type_t upart_type=UP_UNK;
   fat_find_type(disk_car, partition,max_offset,p_fat12,p_fat16,p_fat32,debug,dump_ind,interface,&nbr_offset,&info_offset[0]);
+  /*
+  info_offset[0].fat_type=32;
+  info_offset[0].offset=32;
+  info_offset[0].nbr=1;
+  info_offset[1].fat_type=32;
+  info_offset[1].offset=40;
+  info_offset[1].nbr=921;
+  info_offset[2].fat_type=32;
+  info_offset[2].offset=565;
+  info_offset[2].nbr=1;
+  info_offset[3].fat_type=32;
+  info_offset[3].offset=3064;
+  info_offset[3].nbr=921;
+  info_offset[4].fat_type=32;
+  info_offset[4].offset=3589;
+  info_offset[4].nbr=1;
+  info_offset[5].fat_type=32;
+  info_offset[5].offset=35190;
+  info_offset[5].nbr=1;
+  nbr_offset=6;
+  */
   for(i=0;i<nbr_offset;i++)
   {
     ecrit_rapport("FAT%u at %lu(%u/%u/%u), nbr=%u\n",info_offset[i].fat_type,info_offset[i].offset,
@@ -1548,12 +1583,12 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
     {
       if(info_offset[i].fat_type==info_offset[offset_for_max_nbr].fat_type)
       {
-	if(fat_found==0)
+	if(fat_found==0 && info_offset[i].nbr>=(info_offset[offset_for_max_nbr].nbr+2-1)/2)
 	{
 	  first_fat=i;
 	  fat_found++;
 	}
-	else if(fat_found==1)
+	else if(fat_found==1 && info_offset[i].nbr>=(info_offset[first_fat].nbr+2-1)/2)
 	{
 	  second_fat=i;
 	  fat_found++;
@@ -1561,6 +1596,20 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
       }
     }
     if(fat_found==1)
+    {
+      for(i=first_fat+1;i<nbr_offset;i++)
+      {
+	if(info_offset[i].fat_type==info_offset[offset_for_max_nbr].fat_type)
+	{
+	  if(fat_found==1)
+	  {
+	    second_fat=i;
+	    fat_found++;
+	  }
+	}
+      }
+    }
+    if(fat_found==1)
     {
       switch(upart_type)
       {
@@ -1606,6 +1655,10 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
 	  return UP_UNK;
       }
     }
+    if(debug>0)
+    {
+      ecrit_rapport("first_fat %lu, second_fat %lu\n",info_offset[first_fat].offset, info_offset[second_fat].offset);
+    }
   }
   if(expert>0 && (interface>0))
   {
