@@ -2,7 +2,7 @@
 
     File: dir_fat.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -245,13 +245,13 @@ static t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partiti
   {
     t_file_data *dir_list;
     unsigned int cluster_size=fat_header->cluster_size;
-    unsigned char *buffer_dir=MALLOC(fat_sector_size(fat_header)*cluster_size*5);
+    unsigned char *buffer_dir=MALLOC(fat_sector_size(fat_header)*cluster_size*10);
     unsigned int cluster;
     unsigned int nbr_cluster;
     int stop=0;
-    memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*5);
+    memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*10);
     for(cluster=first_cluster, nbr_cluster=0;
-	((cluster&0x0ff8)!=(unsigned)FAT12_EOC) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
+	((cluster&0x0ff8)!=(unsigned)FAT12_EOC) && (cluster>=2) && (nbr_cluster<10) && (stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT12,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length)+(get_dir_entries(fat_header)*32+fat_sector_size(fat_header)-1)/fat_sector_size(fat_header)+(cluster-2)*cluster_size)*fat_sector_size(fat_header);
@@ -290,14 +290,14 @@ static t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partiti
   {
     t_file_data *dir_list=NULL;
     unsigned int cluster_size=fat_header->cluster_size;
-    unsigned char *buffer_dir=MALLOC(disk_car->sector_size*cluster_size*5);
+    unsigned char *buffer_dir=MALLOC(disk_car->sector_size*cluster_size*10);
     unsigned int cluster;
     unsigned int nbr_cluster;
     int stop=0;
-    memset(buffer_dir,0,disk_car->sector_size*cluster_size*5);
+    memset(buffer_dir,0,disk_car->sector_size*cluster_size*10);
     /* Need to correct the test */
     for(cluster=first_cluster, nbr_cluster=0;
-	((cluster&0xfff8)!=(unsigned)FAT16_EOC) && (cluster>=2) && (nbr_cluster<5)&&(stop==0);
+	((cluster&0xfff8)!=(unsigned)FAT16_EOC) && (cluster>=2) && (nbr_cluster<10)&&(stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT16,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length)+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size+(cluster-2)*cluster_size)*disk_car->sector_size;
@@ -339,14 +339,14 @@ static t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partiti
   {
     t_file_data *dir_list;
     unsigned int cluster_size=fat_header->cluster_size;
-    unsigned char *buffer_dir=MALLOC(fat_sector_size(fat_header)*cluster_size*5);
+    unsigned char *buffer_dir=MALLOC(fat_sector_size(fat_header)*cluster_size*10);
     unsigned int cluster;
     unsigned int nbr_cluster;
     int stop=0;
-    memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*5);
+    memset(buffer_dir,0,fat_sector_size(fat_header)*cluster_size*10);
     /* Need to correct the test */
     for(cluster=(first_cluster==0?le32(fat_header->root_cluster):first_cluster), nbr_cluster=0;
-	((cluster&0xffffff8)!=(unsigned)FAT32_EOC) && (cluster>=2) && (nbr_cluster<5) && (stop==0);
+	((cluster&0xffffff8)!=(unsigned)FAT32_EOC) && (cluster>=2) && (nbr_cluster<10) && (stop==0);
 	cluster=get_next_cluster(disk_car,partition, UP_FAT32,le16(fat_header->reserved), cluster), nbr_cluster++)
     {
       uint64_t start=partition->part_offset+(uint64_t)(le16(fat_header->reserved)+fat_header->fats*le32(fat_header->fat32_length)+(cluster-2)*cluster_size)*fat_sector_size(fat_header);
@@ -442,6 +442,7 @@ int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, const t_partiti
       ecrit_rapport("Not a valid type (upart_type=%u)\n",partition->upart_type);
       return -1;
   }
+  dir_data.copy_file=NULL;
   strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
   return dir_partition(disk_car,partition,&dir_data,0,current_cmd);
 }
