@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -30,7 +30,7 @@
 #include "photorec.h"
 #include "common.h"
 
-static int header_check_crw(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_crw(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 static int data_check_crw(const unsigned char *buffer, const unsigned int buffer_size, t_file_recovery *file_recovery);
 const t_file_hint file_hint_crw= {
   .extension="crw",
@@ -38,17 +38,18 @@ const t_file_hint file_hint_crw= {
   .min_header_distance=0,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=0,
   .recover=1,
   .header_check=&header_check_crw,
   .file_check=NULL,
   .data_check=&data_check_crw
 };
 
-static int header_check_crw(const unsigned char*buffer, const unsigned int buffer_size)
+static const char* header_check_crw(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
-  return(((buffer[0]==0x49 && buffer[1]==0x49)||(buffer[0]==0x4D && buffer[1]==0x4D))
-      && memcmp(buffer+6,"HEAPCCDR",8)==0);
+  if(((buffer[0]==0x49 && buffer[1]==0x49)||(buffer[0]==0x4D && buffer[1]==0x4D))
+      && memcmp(buffer+6,"HEAPCCDR",8)==0)
+    return file_hint_crw.extension;
+  return NULL;
 }
 
 static int data_check_crw(const unsigned char *buffer, const unsigned int buffer_size, t_file_recovery *file_recovery)
