@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -29,26 +29,27 @@
 #include "types.h"
 #include "photorec.h"
 
-static int header_check_dsc(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_dsc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 const t_file_hint file_hint_dsc= {
   .extension="dsc",
   .description="Nikon dsc",
   .min_header_distance=0,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=0,
   .recover=0,
   .header_check=&header_check_dsc,
   .data_check=NULL,
   .file_check=NULL
 };
 
-static int header_check_dsc(const unsigned char *buffer, const unsigned int buffer_size)
+static const char* header_check_dsc(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
   const unsigned char dsc_header[3]= { 'M','L','T'};
   if(buffer_size<1024)
   {
     return 0;
   }
-  return (memcmp(&buffer[588],dsc_header,sizeof(dsc_header))==0);
+  if(memcmp(&buffer[588],dsc_header,sizeof(dsc_header))==0)
+    return file_hint_dsc.extension;
+  return NULL;
 }
