@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -29,24 +29,27 @@
 #include "types.h"
 #include "photorec.h"
 
-static int header_check_mpg(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_mpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 const t_file_hint file_hint_mpg= {
   .extension="mpg",
   .description="Moving Picture Experts Group video",
   .min_header_distance=0,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=1,
   .recover=1,
   .header_check=&header_check_mpg,
   .data_check=NULL,
   .file_check=NULL
 };
-static int header_check_mpg(const unsigned char *buffer, const unsigned int buffer_size)
+static const char* header_check_mpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
-  return (buffer[0]==0x00 && buffer[1]==0x00 && buffer[2]==0x01 &&
-      (buffer[3]==0xB0 || buffer[3]==0xB3 || buffer[3]==0xB5 || buffer[3]==0xBA || buffer[3]==0xBB));
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_mpg)
+    return NULL;
   /* 0xBA is the must common value */
+  if(buffer[0]==0x00 && buffer[1]==0x00 && buffer[2]==0x01 &&
+      (buffer[3]==0xB0 || buffer[3]==0xB3 || buffer[3]==0xB5 || buffer[3]==0xBA || buffer[3]==0xBB))
+    return file_hint_mpg.extension;
+  return NULL;
 }
 
 
