@@ -10,13 +10,13 @@
     (at your option) any later version.
   
     This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the impdfied warranty of
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -29,22 +29,23 @@
 #include "types.h"
 #include "photorec.h"
 
-static int header_check_pdf(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_pdf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 const t_file_hint file_hint_pdf= {
   .extension="pdf",
   .description="Portable Document Format",
   .min_header_distance=0,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=0,
   .recover=1,
   .header_check=&header_check_pdf,
   .data_check=NULL,
   .file_check=NULL
 };
 
-static int header_check_pdf(const unsigned char *buffer, const unsigned int buffer_size)
+static const char* header_check_pdf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
   const unsigned char pdf_header[]  = { '%','P','D','F','-','1'};
-  return (memcmp(buffer,pdf_header,sizeof(pdf_header))==0);
+  if(memcmp(buffer,pdf_header,sizeof(pdf_header))==0)
+    return file_hint_pdf.extension;
+  return NULL;
 }
