@@ -10,13 +10,13 @@
     (at your option) any later version.
   
     This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the imtaried warranty of
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -29,22 +29,25 @@
 #include "types.h"
 #include "photorec.h"
 
-static int header_check_tar(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_tar(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 const t_file_hint file_hint_tar= {
   .extension="tar",
   .description="tar archive",
   .min_header_distance=0x200,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=1,
   .recover=1,
   .header_check=&header_check_tar,
   .data_check=NULL,
   .file_check=NULL
 };
 
-static int header_check_tar(const unsigned char *buffer, const unsigned int buffer_size)
+static const char* header_check_tar(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
   const unsigned char tar_header[]    = { 'u','s','t','a','r'};
-  return (memcmp(&buffer[0x101],tar_header,sizeof(tar_header))==0);
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL && file_recovery->file_stat->file_hint==&file_hint_tar)
+    return NULL;
+  if(memcmp(&buffer[0x101],tar_header,sizeof(tar_header))==0)
+    return file_hint_tar.extension;
+  return NULL;
 }
