@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -29,23 +29,24 @@
 #include "types.h"
 #include "photorec.h"
 
-static int header_check_x3f(const unsigned char *buffer, const unsigned int buffer_size);
+static const char* header_check_x3f(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 const t_file_hint file_hint_x3f= {
   .extension="x3f",
   .description="Sigma/Foveon X3 raw picture",
   .min_header_distance=0,
   .min_filesize=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .stream=0,
   .recover=1,
   .header_check=&header_check_x3f,
   .data_check=NULL,
   .file_check=NULL
 };
 
-static int header_check_x3f(const unsigned char *buffer, const unsigned int buffer_size)
+static const char* header_check_x3f(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 { /* http://www.x3f.info/technotes/FileDocs/X3F_Format.pdf */
-  return (memcmp(buffer,"FOVb",4)==0);
+  if(memcmp(buffer,"FOVb",4)==0)
+    return file_hint_x3f.extension;
+  return NULL;
 }
 
 
