@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -41,10 +41,6 @@
 
 static unsigned int get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
 
-unsigned long int CHS2LBA(const t_param_disk *disk_car,const t_CHS*CHS)
-{ return ((unsigned long int)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1;
-}
-
 unsigned long int C_H_S2LBA(const t_param_disk *disk_car,const unsigned int C, const unsigned int H, const unsigned int S)
 { return ((unsigned long int)C*(disk_car->CHS.head+1)+H)*disk_car->CHS.sector+S-1;
 }
@@ -91,6 +87,7 @@ void dup_t_partition(t_partition *dest, const t_partition *src)
   dest->part_type_i386=src->part_type_i386;
   dest->part_type_sun=src->part_type_sun;
   dest->part_type_mac=src->part_type_mac;
+  dest->part_type_xbox=src->part_type_xbox;
   dest->upart_type=src->upart_type;
   dest->status=src->status;
   dest->order=src->order;
@@ -105,11 +102,27 @@ t_list_disk *insert_new_disk(t_list_disk *list_disk, t_param_disk *disk_car)
   if(disk_car==NULL)
     return list_disk;
   {
-    t_list_disk *prev;
-    t_list_disk *new_disk=(t_list_disk *)MALLOC(sizeof(*new_disk));
+    t_list_disk *cur;
+    t_list_disk *prev=NULL;
+    t_list_disk *new_disk;
+    /* Add it at the end if it doesn't already exist */
+    for(cur=list_disk;cur!=NULL;cur=cur->next)
+    {
+      if(cur->disk->device!=NULL && disk_car->device!=NULL)
+      {
+	if(strcmp(cur->disk->device,disk_car->device)==0)
+	{
+	  if(disk_car->clean!=NULL)
+	    disk_car->clean(disk_car);
+	  FREE(disk_car->device);
+	  FREE(disk_car);
+	  return list_disk;
+	}
+      }
+      prev=cur;
+    }
+    new_disk=(t_list_disk *)MALLOC(sizeof(*new_disk));
     new_disk->disk=disk_car;
-    /* Add it at the end */
-    for(prev=list_disk;prev!=NULL && prev->next!=NULL;prev=prev->next);
     if(prev!=NULL)
     {
       prev->next=new_disk;
@@ -156,6 +169,7 @@ t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_e
 	(next->part->part_type_i386==new_element->part->part_type_i386) &&
 	(next->part->part_type_mac==new_element->part->part_type_mac) &&
 	(next->part->part_type_sun==new_element->part->part_type_sun) &&
+	(next->part->part_type_xbox==new_element->part->part_type_xbox) &&
 	(next->part->upart_type==new_element->part->upart_type || new_element->part->upart_type==UP_UNK))
       { /*CGR 2004/05/31*/
 	if(next->part->status==STATUS_DELETED)
@@ -277,6 +291,7 @@ void  partition_reset(t_partition *partition)
   partition->part_type_i386=P_NO_OS;
   partition->part_type_sun=PSUN_UNK;
   partition->part_type_mac=PMAC_UNK;
+  partition->part_type_xbox=PXBOX_UNK;
   partition->upart_type=UP_UNK;
   partition->status=STATUS_DELETED;
   partition->order=NO_ORDER;
@@ -367,3 +382,17 @@ unsigned int get_geometry_from_list_part(const t_param_disk *disk_car, const t_l
   return head_max;
 }
 
+const char *size_to_unit(uint64_t disk_size, char *buffer)
+{
+  if(disk_size<(uint64_t)10*1024)
+    sprintf(buffer,"%u B", (unsigned)disk_size);
+  else if(disk_size<(uint64_t)10*1024*1024)
+    sprintf(buffer,"%u KB / %u KiB", (unsigned)(disk_size/1000), (unsigned)(disk_size/1024));
+  else if(disk_size<(uint64_t)10*1024*1024*1024)
+    sprintf(buffer,"%u MB / %u MiB", (unsigned)(disk_size/1000/1000), (unsigned)(disk_size/1024/1024));
+  else if(disk_size<(uint64_t)10*1024*1024*1024*1024)
+    sprintf(buffer,"%u GB / %u GiB", (unsigned)(disk_size/1000/1000/1000), (unsigned)(disk_size/1024/1024/1024));
+  else
+    sprintf(buffer,"%u TB / %u TiB", (unsigned)(disk_size/1000/1000/1000/1000), (unsigned)(disk_size/1024/1024/1024/1024));
+  return buffer;
+}
