@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -40,7 +40,7 @@ void change_geometry(t_param_disk *disk_car, char ** current_cmd)
   int done = FALSE;
   char def[LINE_LENGTH];
   char response[LINE_LENGTH];
-  int tmp_val;
+  int tmp_val=0;
   int command;
   int default_option=4;
   int modified=0;
@@ -58,6 +58,18 @@ void change_geometry(t_param_disk *disk_car, char ** current_cmd)
     aff_copy(stdscr);
     wmove(stdscr,5,0);
     wdoprintf(stdscr,"%s, sector size=%u\n",disk_car->description(disk_car),disk_car->sector_size);
+    wmove(stdscr,7,0);
+    wdoprintf(stdscr,"Because these numbers change the way that TestDisk looks for partitions");
+    wmove(stdscr,8,0);
+    wdoprintf(stdscr,"and calculates their sizes, it's important to have the correct disk geometry.");
+    wmove(stdscr,9,0);
+    wdoprintf(stdscr,"PC partitionning programs often make partitions end on cylinder boundaries.");
+    wmove(stdscr,11,0);
+    wdoprintf(stdscr,"A partition's CHS values are based on disk translations which make them");
+    wmove(stdscr,12,0);
+    wdoprintf(stdscr,"different than its physical geometry. The most common CHS head values");
+    wmove(stdscr,13,0);
+    wdoprintf(stdscr,"are: 255, 240 and sometimes 16.");
     wmove(stdscr,COMMAND_LINE_Y, COMMAND_LINE_X);
     wclrtoeol(stdscr);
     wrefresh(stdscr);
