@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -49,13 +49,31 @@
 extern const t_arch_fnct arch_mac;
 extern const t_arch_fnct arch_none;
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part,const int align);
-static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part);
 static t_list_part *reduce_structure(t_list_part *list_part);
-static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
 static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd);
 static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part_bad);
+static int interface_part_bad_log(t_param_disk *disk_car,t_list_part *list_part_bad);
 static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder, char **current_cmd);
-static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind, const int interface);
+static void ask_mbr_order_i386(t_param_disk *disk_car,t_list_part *list_part);
+static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
+static unsigned int tab_insert(uint64_t *tab, uint64_t offset, unsigned int tab_nbr);
+/* Optimization */
+static inline void offset2CHS_inline(const t_param_disk *disk_car,const uint64_t offset, t_CHS*CHS);
+static inline uint64_t CHS2offset_inline(const t_param_disk *disk_car,const t_CHS*CHS);
+
+static inline void offset2CHS_inline(const t_param_disk *disk_car,const uint64_t offset, t_CHS*CHS)
+{
+  uint64_t pos=offset/disk_car->sector_size;
+  CHS->sector=(pos%disk_car->CHS.sector)+1;
+  pos/=disk_car->CHS.sector;
+  CHS->head=pos%(disk_car->CHS.head+1);
+  CHS->cylinder=pos/(disk_car->CHS.head+1);
+}
+
+static inline uint64_t CHS2offset_inline(const t_param_disk *disk_car,const t_CHS*CHS)
+{ return (((uint64_t)CHS->cylinder*(disk_car->CHS.head+1)+CHS->head)*disk_car->CHS.sector+CHS->sector-1)*disk_car->sector_size;
+}
+/* Optimization end */
 
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part, const int align)
 {
@@ -65,7 +83,7 @@ static void align_structure(const t_param_disk *disk_car, t_list_part *list_part
   for(element=list_part;element!=NULL;element=element->next)
   {
     t_CHS end;
-    offset2CHS(disk_car,element->part->part_offset+element->part->part_size-1,&end);
+    offset2CHS_inline(disk_car,element->part->part_offset+element->part->part_size-1,&end);
     if(align>0)
     {
       end.sector=disk_car->CHS.sector;
@@ -74,7 +92,7 @@ static void align_structure(const t_param_disk *disk_car, t_list_part *list_part
     {
       end.head=disk_car->CHS.head;
     }
-    element->part->part_size=(uint64_t)CHS2offset(disk_car, &end)-element->part->part_offset+disk_car->sector_size;
+    element->part->part_size=(uint64_t)CHS2offset_inline(disk_car, &end)-element->part->part_offset+disk_car->sector_size;
   }
 }
 
@@ -95,22 +113,45 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
   int quit=0;
   int offset=0;
   int pos_num=0;
+  uint64_t disk_size=disk_car->disk_size;
   t_list_part *pos=list_part;
   if(list_part==NULL)
     return 1;
+  {
+    t_list_part *parts;
+    for(parts=list_part;parts!=NULL;parts=parts->next)
+    {
+      if(disk_size<parts->part->part_offset+parts->part->part_size-1)
+	disk_size=parts->part->part_offset+parts->part->part_size-1;
+    }
+  }
   aff_copy(stdscr);
   wmove(stdscr,4,0);
   wdoprintf(stdscr,"%s",disk_car->description(disk_car));
   wmove(stdscr,6,0);
-  wdoprintf(stdscr,"Check the harddisk size: HD jumpers settings, BIOS detection, OS limitation...");
+  {
+    char buffer_disk_size[100];
+    char buffer_disk_size_found[100];
+    wdoprintf(stdscr,"The harddisk (%s) seems too small! (< %s)",
+	size_to_unit(disk_car->disk_size,buffer_disk_size), size_to_unit(disk_size,buffer_disk_size_found));
+  }
   wmove(stdscr,7,0);
+  wdoprintf(stdscr,"Check the harddisk size: HD jumpers settings, BIOS detection...");
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+  if(disk_car->disk_size<=((uint64_t)1<<(28-1)) && disk_size>=((uint64_t)1<<(28-1)))
+  {
+    wmove(stdscr,8,0);
+    wdoprintf(stdscr,"Hint: update Windows to support LBA48 (minimum: W2K SP4 or XP SP1)");
+  }
+#endif
+  wmove(stdscr,9,0);
   if(list_part->next==NULL)
   {
-    wdoprintf(stdscr,"The following partition can't be recovered. The harddisk seems too small!");
+    wdoprintf(stdscr,"The following partition can't be recovered:");
   } else {
-    wdoprintf(stdscr,"The following partitions can't be recovered. The harddisk seems too small!");
+    wdoprintf(stdscr,"The following partitions can't be recovered:");
   }
-  mvwaddstr(stdscr,8,0,msg_PART_HEADER);
+  mvwaddstr(stdscr,10,0,msg_PART_HEADER);
   wmove(stdscr,22,0);
   wattrset(stdscr,A_STANDOUT);
   wdoprintf(stdscr,"[  Quit  ]");
@@ -123,10 +164,11 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
     for(i=0,parts=list_part;(parts!=NULL) && (i<offset);parts=parts->next,i++);
     for(i=offset;(parts!=NULL) &&((i-offset)<INTER_BAD_PART);i++,parts=parts->next)
     {
-      wmove(stdscr,9+i-offset,0);
+      wmove(stdscr,11+i-offset,0);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
       if(parts==pos)
       {
+	char buffer_part_size[100];
 	wattrset(stdscr,A_STANDOUT);
 	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
 	wattroff(stdscr,A_STANDOUT);
@@ -136,7 +178,7 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
 	{
 	  wdoprintf(stdscr,"%s, ",parts->part->info);
 	}
-	wdoprintf(stdscr,"%lu MB",(long unsigned)(parts->part->part_size/(1024*1024)));
+	wdoprintf(stdscr,"%s",size_to_unit(parts->part->part_size,buffer_part_size));
       } else
       {
 	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
@@ -215,6 +257,47 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
   return 0;
 }
 
+static int interface_part_bad_log(t_param_disk *disk_car,t_list_part *list_part)
+{
+  uint64_t disk_size=disk_car->disk_size;
+  if(list_part==NULL)
+    return 1;
+  {
+    t_list_part *parts;
+    for(parts=list_part;parts!=NULL;parts=parts->next)
+    {
+      if(disk_size<parts->part->part_offset+parts->part->part_size-1)
+	disk_size=parts->part->part_offset+parts->part->part_size-1;
+    }
+  }
+  ecrit_rapport("%s\n",disk_car->description(disk_car));
+  ecrit_rapport("Check the harddisk size: HD jumpers settings, BIOS detection...\n");
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+  if(disk_car->disk_size<=((uint64_t)1<<(28-1)) && disk_size>=((uint64_t)1<<(28-1)))
+  {
+    ecrit_rapport("Hint: update Windows to support LBA48 (minimum: W2K SP4 or XP SP1)");
+  }
+#endif
+  {
+    char buffer_disk_size[100];
+    char buffer_disk_size_found[100];
+    ecrit_rapport("The harddisk (%s) seems too small! (< %s)\n",
+	size_to_unit(disk_car->disk_size,buffer_disk_size), size_to_unit(disk_size,buffer_disk_size_found));
+  }
+  if(list_part->next==NULL)
+  {
+    ecrit_rapport("The following partition can't be recovered:\n");
+  } else {
+    ecrit_rapport("The following partitions can't be recovered:\n");
+  }
+  {
+    t_list_part *parts;
+    for(parts=list_part;parts!=NULL;parts=parts->next)
+      aff_part_rapport(disk_car,parts->part);
+  }
+  return 0;
+}
+
 static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder, char**current_cmd)
 {
   int quit=0;
@@ -227,7 +310,7 @@ static void warning_geometry(t_param_disk *disk_car, const unsigned int recomman
   wmove(stdscr,7,0);
   wdoprintf(stdscr,"but the correct value may be %u.",recommanded_heads_per_cylinder);
   wmove(stdscr,8,0);
-  wdoprintf(stdscr,"Use the Geometry menu to change this value.");
+  wdoprintf(stdscr,"You can use the Geometry menu to change this value.");
   wmove(stdscr,22,0);
   wattrset(stdscr,A_STANDOUT);
   wdoprintf(stdscr,"[  Quit  ]");
@@ -257,93 +340,43 @@ static void warning_geometry(t_param_disk *disk_car, const unsigned int recomman
   } while(quit==0);
 }
 
-static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind, const int interface)
+static unsigned int tab_insert(uint64_t *tab, uint64_t offset, unsigned int tab_nbr)
 {
-  t_list_part *element;
-  unsigned int test_current=0;
-  unsigned int test_total=0;
-  int ind_stop=0;
-  for(element=list_part_bad;element!=NULL;element=element->next)
+  if(tab_nbr<MAX_SEARCH_LOCATION-1)
   {
-    if(element->part->upart_type!=UP_RAID)
-      test_total+=6;
+    unsigned int i,j;
+    for(i=0;i<tab_nbr && tab[i]<=offset;i++);
+    tab_nbr++;
+    for(j=tab_nbr;j>i;j--)
+      tab[j]=tab[j-1];
+    tab[i]=offset;
   }
-  if(interface)
-  {
-    aff_copy(stdscr);
-    wmove(stdscr,4,0);
-    wdoprintf(stdscr,"%s",disk_car->description(disk_car));
-    wmove(stdscr,22,0);
-    wattrset(stdscr,A_STANDOUT);
-    waddstr(stdscr,"  Stop  ");
-    wattroff(stdscr,A_STANDOUT);
-  }
-  /* Little trick to detect Linux software Raid quickly */
-  for(element=list_part_bad;element!=NULL && ind_stop==0;element=element->next)
-  {
-    if(element->part->upart_type!=UP_RAID)
-    {
-      unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-      struct mdp_superblock_s *sb=(struct mdp_superblock_s*)&buffer;
-      unsigned int disk_factor;
-      unsigned int help_factor;
-      if(debug>1)
-      {
-	ecrit_rapport("search_part_raid:\n");
-	aff_part_rapport(disk_car,element->part);
-      }
-      for(disk_factor=6;disk_factor>=1 && ind_stop==0;disk_factor--)
-      { /* disk_factor=1, detect Raid 0/1 */
-        /* disk_factor>1, detect Raid 5 */
-	if(interface)
-	{
-	  wmove(stdscr,6,0);
-	  wdoprintf(stdscr,"Searching for software Raid partition: %02d%%",test_current*100/test_total);
-	  wrefresh(stdscr);
-	  ind_stop|=check_enter_or_s(stdscr);
-	}
-	test_current++;
-	for(help_factor=0;help_factor<=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES+3 && ind_stop==0;help_factor++)
-	{
-	  uint64_t offset=(uint64_t)MD_NEW_SIZE_SECTORS((element->part->part_size/disk_factor+help_factor*MD_RESERVED_BYTES-1)/MD_RESERVED_BYTES*MD_RESERVED_BYTES/512)*512;
-	  if(debug>1)
-	  {
-	    ecrit_rapport("search_part_raid offset=%llu\n",(long long unsigned)offset);
-	  }
-	  if(disk_car->read(disk_car,sizeof(buffer), &buffer, element->part->part_offset+offset)==0)
-	  {
-	    t_partition *new_partition=partition_new();
-	    dup_t_partition(new_partition,element->part);
-	    if(test_MD(disk_car,sb,new_partition,debug,dump_ind)==0)
-	    {
-	      set_MD_info(disk_car,sb,new_partition,debug,dump_ind);
-	      new_partition->part_type_i386=P_RAID;
-	      new_partition->part_type_sun=PSUN_RAID;
-	      new_partition->part_type_mac=PMAC_LINUX;
-	      new_partition->part_size=offset+MD_RESERVED_BYTES;
-	      aff_part_rapport(disk_car,new_partition);
-	      list_part=insert_new_partition(list_part,new_partition);
-	    }
-	    else
-	      FREE(new_partition);
-	  }
-	}
-      }
-    }
-  }
-  return list_part;
+  return tab_nbr;
 }
 
-t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int debug, const int dump_ind, const int fast_mode, const int interface)
+t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump_ind, const int fast_mode, const int interface)
 {
-  int ind_stop=0;
   uint64_t try_offset[MAX_SEARCH_LOCATION];
+  uint64_t try_offset_raid[MAX_SEARCH_LOCATION];
   unsigned int try_offset_nbr=0;
+  unsigned int try_offset_raid_nbr=0;
   unsigned int old_permil=0;
+  int ind_stop=0;
   t_CHS start;
   t_list_part *list_part=NULL;
   t_list_part *list_part_bad=NULL;
   t_partition *partition=partition_new();
+#ifdef NEWBIE
+  t_param_disk *disk_car_geo16=(t_param_disk*)MALLOC(sizeof(*disk_car_geo16));
+  t_param_disk *disk_car_geo255=(t_param_disk*)MALLOC(sizeof(*disk_car_geo255));
+  disk_car_geo16->sector_size=disk_car->sector_size;
+  disk_car_geo16->CHS.sector=disk_car->CHS.sector;
+  disk_car_geo16->CHS.head=16-1;
+  disk_car_geo255->sector_size=disk_car->sector_size;
+  disk_car_geo255->CHS.sector=disk_car->CHS.sector;
+  disk_car_geo255->CHS.head=255-1;
+#endif
+
   if(interface!=0)
   {
     wmove(stdscr,22,0);
@@ -359,99 +392,137 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
   {
     if(interface!=0)
     {
-      unsigned int permil=start.cylinder*1000/(disk_car->CHS.cylinder>0?disk_car->CHS.cylinder:1);
+      unsigned int permil=start.cylinder*1000/(disk_car->CHS.cylinder+1);
       if(permil>old_permil || disk_car->arch!=&arch_mac)
       {
 	wmove(stdscr,ANALYSE_Y,ANALYSE_X);
 	wclrtoeol(stdscr);
-	wdoprintf(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,start.cylinder*100/(disk_car->CHS.cylinder>0?disk_car->CHS.cylinder:1));
+	wdoprintf(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,start.cylinder*100/(disk_car->CHS.cylinder+1));
 	wrefresh(stdscr);
 	old_permil=permil;
       }
     }
     for(;(start.head<=disk_car->CHS.head)&&(ind_stop==0);start.head++)
     {
-      unsigned int sector_inc;
-      int part_offset;
+      unsigned int sector_inc=1;
+      int test_nbr=0;
       if(interface!=0)
       {
 	ind_stop|=check_enter_or_s(stdscr);
       }
-      for(start.sector=1,sector_inc=1,part_offset=0;(start.sector<=disk_car->CHS.sector);start.sector+=sector_inc)
+      for(start.sector=1;start.sector<=disk_car->CHS.sector;start.sector+=sector_inc)
       {
 	int res=0;
 	unsigned int i;
-	int search_now=0;
-	partition_reset(partition);	/* must not reset lba */
-	partition->part_offset=CHS2offset(disk_car,&start);
-	for(i=0;i<try_offset_nbr;i++)
+#ifdef NEWBIE
+	t_CHS start_geo16;
+	t_CHS start_geo255;
+#endif
+	partition->part_size=(uint64_t)0;
+	partition->part_offset=CHS2offset_inline(disk_car,&start);
+#ifdef NEWBIE
+	offset2CHS_inline(disk_car_geo16,partition->part_offset,&start_geo16);
+	offset2CHS_inline(disk_car_geo255,partition->part_offset,&start_geo255);
+#endif
+	if(test_nbr==0)
 	{
-	  if(try_offset[i]==partition->part_offset)
-	    search_now=1;
-	}
-	if((start.sector==1) && ((start.head<=2)||(fast_mode==2)||(search_now>0)))
-	{
-	  part_offset++;
-	  switch(part_offset)
+	  int search_now=0;
+	  sector_inc=0;
+	  i=0;
+	  while(try_offset_nbr>0 && try_offset[0]<=partition->part_offset)
 	  {
-	    case 1:
-	      sector_inc=0;
-	      res=search_type_64(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 2:
-	      res=search_type_2(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 3:
-	      res=search_type_1(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 4:
-	      /* Try to catch disklabel before BSD FFS partition */
-	      res=search_type_128(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 5:
-	      /* Try to catch disklabel before BSD FFS partition */
-	      res=search_type_16(disk_car,partition,debug,dump_ind);
-	      break;
-	    case 6:
-	      res=search_type_0(disk_car,partition,debug,dump_ind);
-	      sector_inc=1;
-	      break;
-	    default:
-	      ecrit_rapport("\nBUG: unusual value of part_offset\n");
-	      break;
+	    unsigned int j;
+	    if(try_offset[0]==partition->part_offset)
+	      search_now=1;
+	    for(j=0;j<try_offset_nbr-1;j++)
+	      try_offset[j]=try_offset[j+1];
+	    try_offset_nbr--;
+	  }
+	  search_now|= (start.sector==1 && fast_mode>1) || (start.sector==1 && start.head<=2)
+#ifdef NEWBIE
+	    || (start_geo16.sector==1 && fast_mode>1) || (start_geo16.sector==1 && start_geo16.head<=2)
+	    || (start_geo255.sector==1 && fast_mode>1) || (start_geo255.sector==1 && start_geo255.head<=2)
+#endif
+	    ;
+	  if(search_now==0)
+	    test_nbr=6;
+	  else
+	  {
+	    res=search_type_64(disk_car,partition,debug,dump_ind);
+	    test_nbr++;
 	  }
 	}
-	else if(start.sector==7 && fast_mode>0 && ((start.head<=2)||(fast_mode==2)||(search_now>0)))
+	if(res<=0 && test_nbr==1)
 	{
-	  res=search_FAT_backup(disk_car,partition,debug,dump_ind);
+	  res=search_type_2(disk_car,partition,debug,dump_ind);
+	  test_nbr++;
 	}
-	else if(start.sector==disk_car->CHS.sector && (fast_mode>0))
+	if(res<=0 && test_nbr==2)
 	{
-	  if((start.head==disk_car->CHS.head) || (fast_mode>1) || (search_now>0))
+	    res=search_type_1(disk_car,partition,debug,dump_ind);
+	    test_nbr++;
+	}
+	if(res<=0 && test_nbr==3)
+	{
+	    /* Try to catch disklabel before BSD FFS partition */
+	    res=search_type_128(disk_car,partition,debug,dump_ind);
+	    test_nbr++;
+	}
+	if(res<=0 && test_nbr==4)
+	{
+	    /* Try to catch disklabel before BSD FFS partition */
+	    res=search_type_16(disk_car,partition,debug,dump_ind);
+	    test_nbr++;
+	}
+	if(res<=0 && test_nbr==5)
+	{
+	  res=search_type_0(disk_car,partition,debug,dump_ind);
+	  test_nbr++;
+	}
+	if(res<=0 && test_nbr==6)
+	{
+	  if(fast_mode==0)
+	    test_nbr=10;
+	  else
 	  {
+	    if((start.sector==7 && (start.head<=2 || fast_mode>1))
+#ifdef NEWBIE
+		|| (start_geo16.sector==7 && (start_geo16.head<=2 || fast_mode>1))
+		|| (start_geo255.sector==7 && (start_geo255.head<=2 || fast_mode>1))
+#endif
+	      )
+	      res=search_FAT_backup(disk_car,partition,debug,dump_ind);
+	    test_nbr++;
+	  }
+	}
+	if(res<=0 && test_nbr==7)
+	{
+	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1))
+#ifdef NEWBIE
+	    || (start_geo16.sector==disk_car->CHS.sector && (start_geo16.head==disk_car->CHS.head || fast_mode>1))
+	    || (start_geo255.sector==disk_car->CHS.sector && (start_geo255.head==disk_car->CHS.head || fast_mode>1))
+#endif
+	    )
 	    res=search_NTFS_backup(disk_car,partition,debug,dump_ind);
-	  }
+	  test_nbr++;
 	}
-	else if(start.sector==disk_car->CHS.sector-1 && (fast_mode>0))
+	if(res<=0 && test_nbr==8)
 	{
-	  if((start.head==disk_car->CHS.head) || (fast_mode>1) || (search_now>0))
-	  {
+	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1))
+#ifdef NEWBIE
+	    || (start_geo16.sector==disk_car->CHS.sector && (start_geo16.head==disk_car->CHS.head || fast_mode>1))
+	    || (start_geo255.sector==disk_car->CHS.sector && (start_geo255.head==disk_car->CHS.head || fast_mode>1))
+#endif
+	    )
 	    res=search_HFS_backup(disk_car,partition,debug,dump_ind);
-	  }
+	  test_nbr++;
 	}
-	if(res<0)
-	{
-	  if(interface!=0)
-	  {
-	    wdoprintf(stdscr,msg_READ_ERROR_AT, start.cylinder,start.head,start.sector,(unsigned long)(partition->part_offset/disk_car->sector_size));
-	  }
-	}
-	if(res<=0 && paranoid!=0)
+	if(res<=0 && test_nbr==9)
 	{
 	  int s_log_block_size;
 	  uint64_t ref_offset=partition->part_offset;
 	  /* try backup superblock */
-	  /* It must be in paranoid mode because it can hide otherwise other partition type */
+	  /* It must be in fast_mode>0 because it can hide otherwise other partition type */
 	  /* Block size: 1024, 2048 or 4096 bytes (8192 bytes on Alpha systems) */
 	  /* From e2fsprogs-1.34/lib/ext2fs/initialize.c: set_field(s_first_data_block, super->s_log_block_size ? 0 : 1); */
 	  /* Assumes that TestDisk is not running under Alpha and s_blocks_per_group=8 * block size */
@@ -460,49 +531,92 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	    /* sparse superblock feature: The groups chosen are 0, 1 and powers of 3, 5 and 7. */
 	    /* Checking group 3 */
 	    const uint64_t hd_offset=3*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)*8*(EXT2_MIN_BLOCK_SIZE<<s_log_block_size)+(s_log_block_size==0?2*DEFAULT_SECTOR_SIZE:0);
-	    if(ref_offset>=hd_offset &&
-	     offset2sector(disk_car,ref_offset-hd_offset)==1 &&  
-	     (offset2head(disk_car,ref_offset-hd_offset)<=2 || fast_mode==2 || (search_now>0 && fast_mode>0)))
+	    if(ref_offset>=hd_offset)
 	    {
-	      unsigned char buffer[2*DEFAULT_SECTOR_SIZE];
-	      if(disk_car->read(disk_car,sizeof(buffer), &buffer, ref_offset)==0)
+	      t_CHS start_ext2;
+#ifdef NEWBIE
+	      t_CHS start_ext2_geo16;
+	      t_CHS start_ext2_geo255;
+#endif
+	      offset2CHS_inline(disk_car,ref_offset-hd_offset,&start_ext2);
+#ifdef NEWBIE
+	      offset2CHS_inline(disk_car_geo16,partition->part_offset,&start_ext2_geo16);
+	      offset2CHS_inline(disk_car_geo255,partition->part_offset,&start_ext2_geo255);
+#endif
+	      if((start_ext2.sector==1 &&  (start_ext2.head<=2 || fast_mode>1))
+#ifdef NEWBIE
+		|| (start_ext2_geo16.sector==1 &&  (start_ext2_geo16.head<=2 || fast_mode>1))
+		|| (start_ext2_geo255.sector==1 &&  (start_ext2_geo255.head<=2 || fast_mode>1))
+#endif
+		)
 	      {
-		const struct ext2_super_block *sb=(const struct ext2_super_block*)&buffer;
-		if(le16(sb->s_block_group_nr)>0)
+		unsigned char buffer[2*DEFAULT_SECTOR_SIZE];
+		if(disk_car->read(disk_car,sizeof(buffer), &buffer, ref_offset)==0)
 		{
-		  if(recover_EXT2(disk_car,sb,partition,debug,dump_ind)==0)
-		    res=1;
+		  const struct ext2_super_block *sb=(const struct ext2_super_block*)&buffer;
+		  if(le16(sb->s_block_group_nr)>0)
+		  {
+		    if(recover_EXT2(disk_car,sb,partition,debug,dump_ind)==0)
+		      res=1;
+		  }
+		}
+		else
+		{
+		  res=-1;
 		}
-	      }
-	      else
-	      {
-		res=-1;
 	      }
 	    }
 	  }
+	  test_nbr++;
 	}
-	if(res<=0 && fast_mode>1)
+	if(res<=0 && test_nbr==10)
 	{
-	  /* Because this superblock is at the end of the partition, you should be
-	   * in slow mode to find it... */
-	  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-	  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+	  int search_now=0;
+	  while(try_offset_raid_nbr>0 && try_offset_raid[0]<=partition->part_offset)
 	  {
-	    res=-1;
+	    unsigned int j;
+	    if(try_offset_raid[0]==partition->part_offset)
+	      search_now=1;
+	    for(j=0;j<try_offset_raid_nbr-1;j++)
+	      try_offset_raid[j]=try_offset_raid[j+1];
+	    try_offset_raid_nbr--;
 	  }
-	  else
-	  {
-	    if(recover_MD(disk_car,(const struct mdp_superblock_s*)&buffer,partition,debug,dump_ind)==0)
-	      res=1;
+	  if(fast_mode>1 || search_now>0)
+	  { /* Search Linux software RAID */
+	    unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+	    if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+	    {
+	      res=-1;
+	    }
 	    else
-	      res=0;
+	    {
+	      if(recover_MD(disk_car,(const struct mdp_superblock_s*)&buffer,partition,debug,dump_ind)==0)
+		res=1;
+	      else
+		res=0;
+	    }
+	  }
+	  test_nbr++;
+	}
+	if(test_nbr==11)
+	{
+	  sector_inc=1;
+	  test_nbr=0;
+	}
+	if(res<0)
+	{
+	  if(interface!=0)
+	  {
+	    wmove(stdscr,ANALYSE_Y+1,ANALYSE_X);
+	    wclrtoeol(stdscr);
+	    wdoprintf(stdscr,msg_READ_ERROR_AT, start.cylinder,start.head,start.sector,(unsigned long)(partition->part_offset/disk_car->sector_size));
 	  }
 	}
-	if(res>0)
+	else if(res>0)
 	{
 	  partition->status=STATUS_DELETED;
 	  aff_part_rapport(disk_car,partition);
-	  aff_part_buffer(AFF_PART_ORDER,disk_car,partition);
+	  aff_part_buffer(AFF_PART_SHORT,disk_car,partition);
 	  if(interface)
 	  {
 	    aff_buffer(BUFFER_SHOW,"Q");
@@ -511,6 +625,38 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	  {
 	    uint64_t pos_fin;
 	    pos_fin=partition->part_offset+partition->part_size-1;
+	    /* */
+	    if(partition->upart_type!=UP_RAID)
+	    {
+	      unsigned int disk_factor;
+	      unsigned int help_factor;
+	      unsigned int help_factor_max;
+	      const int align=2;
+	      t_CHS end;
+	      offset2CHS_inline(disk_car,partition->part_offset+partition->part_size-1,&end);
+	      if(align>0)
+	      {
+		end.sector=disk_car->CHS.sector;
+	      }
+	      if(align>1)
+	      {
+		end.head=disk_car->CHS.head;
+	      }
+	      help_factor_max=((uint64_t)CHS2offset_inline(disk_car, &end)-partition->part_offset+disk_car->sector_size-partition->part_size)/MD_RESERVED_BYTES;
+	      if(help_factor_max<3)
+		help_factor_max=3;
+	      help_factor_max+=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES;
+	      for(disk_factor=6;disk_factor>=1 && ind_stop==0;disk_factor--)
+	      { /* disk_factor=1, detect Raid 0/1 */
+		/* disk_factor>1, detect Raid 5 */
+		for(help_factor=0;help_factor<=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES+3 && ind_stop==0;help_factor++)
+		{
+		  uint64_t offset=(uint64_t)MD_NEW_SIZE_SECTORS((partition->part_size/disk_factor+help_factor*MD_RESERVED_BYTES-1)/MD_RESERVED_BYTES*MD_RESERVED_BYTES/512)*512;
+		  try_offset_raid_nbr=tab_insert(try_offset_raid,partition->part_offset+offset,try_offset_raid_nbr);
+		}
+	      }
+	    }
+	    /* */
 	    if(pos_fin<=disk_car->disk_size)
 	    {
 	      {
@@ -522,30 +668,18 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	      {
 		uint64_t next_part_offset=partition->part_offset+partition->part_size-1+1;
 		uint64_t head_size=disk_car->CHS.sector*disk_car->sector_size;
-		if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
-		{
-		  try_offset[try_offset_nbr++]=next_part_offset;
-		}
-		if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
-		{
-		  try_offset[try_offset_nbr++]=next_part_offset+head_size;
-		}
+		try_offset_nbr=tab_insert(try_offset,next_part_offset,try_offset_nbr);
+		try_offset_nbr=tab_insert(try_offset,next_part_offset+head_size,try_offset_nbr);
 		if(next_part_offset%head_size!=0)
 		{
-		  if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
-		  {
-		    try_offset[try_offset_nbr++]=(next_part_offset+head_size-1)/head_size*head_size;
-		  }
-		  if(try_offset_nbr<MAX_SEARCH_LOCATION-1)
-		  {
-		    try_offset[try_offset_nbr++]=(next_part_offset+head_size-1)/head_size*head_size+head_size;
-		  }
+		  try_offset_nbr=tab_insert(try_offset,(next_part_offset+head_size-1)/head_size*head_size,try_offset_nbr);
+		  try_offset_nbr=tab_insert(try_offset,(next_part_offset+head_size-1)/head_size*head_size+head_size,try_offset_nbr);
 		}
 	      }
-	      if((paranoid==0) && (partition->part_offset+partition->part_size-disk_car->sector_size> CHS2offset(disk_car,&start)))
+	      if((fast_mode==0) && (partition->part_offset+partition->part_size-disk_car->sector_size> CHS2offset_inline(disk_car,&start)))
 	      {
-		offset2CHS(disk_car,partition->part_offset+partition->part_size-1,&start);
-		part_offset=0;
+		offset2CHS_inline(disk_car,partition->part_offset+partition->part_size-1,&start);
+		test_nbr=0;
 		sector_inc=1;
 	      }
 	    }
@@ -575,21 +709,25 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	      ecrit_rapport("Partition not added.\n");
 	    }
 	  }
+	  partition_reset(partition);
 	}
       }
     }
   }
   FREE(partition);
-  list_part=search_part_raid(disk_car,list_part,list_part_bad,debug,dump_ind,interface);
-  if(interface!=0 && list_part_bad!=NULL)
+  if(ind_stop>0)
+    ecrit_rapport("Search for partition aborted\n");
+  if(list_part_bad!=NULL)
   {
-    interface_part_bad(disk_car,list_part_bad);
+    interface_part_bad_log(disk_car,list_part_bad);
+    if(interface!=0)
+      interface_part_bad(disk_car,list_part_bad);
   }
   delete_list_part(list_part_bad);
   return list_part;
 }
 
-static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part)
+static void ask_mbr_order_i386(t_param_disk *disk_car,t_list_part *list_part)
 {
   t_partition *table[4];
   t_partition *table2[4];
@@ -702,9 +840,8 @@ static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part)
   } while(res!=0 || quit==0);
 }
 
-
 static t_list_part *reduce_structure(t_list_part *list_part)
-{ 
+{
   t_list_part *element=list_part;
   t_list_part *prev=NULL;
   while(element)
@@ -782,7 +919,7 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
     }
     if(deb->prev==NULL)
     {
-      offset2CHS(disk_car,deb->part->part_offset-1,&start);
+      offset2CHS_inline(disk_car,deb->part->part_offset-1,&start);
       if((align>0) && (start.cylinder>0||start.head>1))
       {
 	start.cylinder=0;
@@ -795,9 +932,9 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
       start.cylinder=offset2cylinder(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1)+1;
       start.head=0;
       start.sector=1;
-      if(CHS2offset(disk_car,&start)>=deb->part->part_offset)
+      if(CHS2offset_inline(disk_car,&start)>=deb->part->part_offset)
       {
-	offset2CHS(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
+	offset2CHS_inline(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
 	start.sector=1;
 	start.head++;
 	if(start.head>=disk_car->CHS.head)
@@ -805,9 +942,9 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
 	  start.cylinder++;
 	  start.head=0;
 	}
-	if(CHS2offset(disk_car,&start)>=deb->part->part_offset)
+	if(CHS2offset_inline(disk_car,&start)>=deb->part->part_offset)
 	{
-	  offset2CHS(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
+	  offset2CHS_inline(disk_car,deb->prev->part->part_offset+deb->prev->part->part_size-1+1,&start);
 	}
       }
     }
@@ -822,9 +959,9 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
       end.cylinder=offset2cylinder(disk_car,fin->next->part->part_offset)-1; /* 8 october 2002 */
       end.head=disk_car->CHS.head;
       end.sector=disk_car->CHS.sector;
-      if(CHS2offset(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
+      if(CHS2offset_inline(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
       {
-	offset2CHS(disk_car,fin->next->part->part_offset-1,&end);
+	offset2CHS_inline(disk_car,fin->next->part->part_offset-1,&end);
 	end.sector=disk_car->CHS.sector;
 	if(end.head>0)
 	  end.head--;
@@ -833,9 +970,9 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
 	  end.cylinder--;
 	  end.head=disk_car->CHS.head;
 	}
-	if(CHS2offset(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
+	if(CHS2offset_inline(disk_car,&end)<=fin->part->part_offset+fin->part->part_size-1)
 	{
-	  offset2CHS(disk_car,fin->next->part->part_offset-1,&end);
+	  offset2CHS_inline(disk_car,fin->next->part->part_offset-1,&end);
 	}
       }
     }
@@ -846,36 +983,36 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
     {
       ecrit_rapport("add_ext_part_i386: min\n");
     }
-    offset2CHS(disk_car,deb->part->part_offset-1,&start);
+    offset2CHS_inline(disk_car,deb->part->part_offset-1,&start);
     start.sector=1;
-    if(deb->prev && CHS2offset(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
+    if(deb->prev && CHS2offset_inline(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
     {
-      offset2CHS(disk_car,deb->part->part_offset-1,&start);
+      offset2CHS_inline(disk_car,deb->part->part_offset-1,&start);
       start.sector=1;
-      if(CHS2offset(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
+      if(CHS2offset_inline(disk_car,&start)<=deb->prev->part->part_offset+deb->prev->part->part_size-1)
       {
-	offset2CHS(disk_car,deb->part->part_offset-1,&start);
+	offset2CHS_inline(disk_car,deb->part->part_offset-1,&start);
       }
     }
-    offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
+    offset2CHS_inline(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
     end.head=disk_car->CHS.head;
     end.sector=disk_car->CHS.sector;
-    if(fin->next && CHS2offset(disk_car,&end)>=fin->next->part->part_offset)
+    if(fin->next && CHS2offset_inline(disk_car,&end)>=fin->next->part->part_offset)
     {
-      offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
+      offset2CHS_inline(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
       end.sector=disk_car->CHS.sector;
     }
-    if(fin->next && CHS2offset(disk_car,&end)>=fin->next->part->part_offset)
+    if(fin->next && CHS2offset_inline(disk_car,&end)>=fin->next->part->part_offset)
     {
-      offset2CHS(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
+      offset2CHS_inline(disk_car,fin->part->part_offset+fin->part->part_size-1,&end);
     }
   }
   new_partition=partition_new();
   new_partition->order=order;
   new_partition->part_type_i386=(end.cylinder>1023?P_EXTENDX:P_EXTENDED);
   new_partition->status=STATUS_EXT;
-  new_partition->part_offset=CHS2offset(disk_car,&start);
-  new_partition->part_size=(uint64_t)CHS2offset(disk_car,&end)-new_partition->part_offset+disk_car->sector_size;
+  new_partition->part_offset=CHS2offset_inline(disk_car,&start);
+  new_partition->part_size=(uint64_t)CHS2offset_inline(disk_car,&end)-new_partition->part_offset+disk_car->sector_size;
   return insert_new_partition(list_part, new_partition);
 }
 
@@ -896,7 +1033,10 @@ static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, cons
 	  fat32_boot_sector(disk_car, element->part, debug, dump_ind, expert,current_cmd);
 	  break;
 	case UP_NTFS:
-	  ntfs_boot_sector(disk_car, element->part, debug, dump_ind,current_cmd);
+	  ntfs_boot_sector(disk_car, element->part, debug, dump_ind, expert, current_cmd);
+	  break;
+	case UP_HFSP:
+	  HFSP_boot_sector(disk_car, element->part, debug, dump_ind, expert, current_cmd);
 	  break;
 	default:
 	  ecrit_rapport("Need to fix\n");
@@ -908,35 +1048,32 @@ static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, cons
   return 0;
 }
 
-int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align, const int ask_part_order, const unsigned int expert, char **current_cmd)
+int interface_recovery(t_param_disk *disk_car, const int debug, const int dump_ind, const int align, const int ask_part_order, const unsigned int expert, char **current_cmd)
 {
   int res_interface_write;
   int menu=-1;
+  int fast_mode=0;
   do
   {
     t_list_part *list_part;
     t_list_part *element;
-    int can_search_deeper=1;
     aff_copy(stdscr);
     wmove(stdscr,4,0);
     wdoprintf(stdscr,"%s",disk_car->description(disk_car));
     wmove(stdscr,5,0);
     res_interface_write=0;
-    if(paranoid==1 && (fast_mode==2 || (fast_mode==1 &&expert==0)))
-      can_search_deeper=0;
-    list_part=search_part(disk_car,paranoid,debug,dump_ind,fast_mode,1);
+    list_part=search_part(disk_car,debug,dump_ind,fast_mode,1);
     if(list_part!=NULL)
     {
       unsigned int head_max;
       head_max=get_geometry_from_list_part(disk_car, list_part, debug);
       if(disk_car->CHS.head!=head_max)
       {
-	warning_geometry(disk_car,head_max+1,current_cmd);
+	if(disk_car->arch != &arch_none || list_part->next!=NULL)
+	  warning_geometry(disk_car,head_max+1,current_cmd);
       }
     }
     align_structure(disk_car,list_part,align);
-    list_part=search_part_raid(disk_car,list_part,list_part,debug,dump_ind,1);
-    align_structure(disk_car,list_part,align);
 #ifdef DEBUG
     check_list_part(list_part);
 #endif
@@ -1007,13 +1144,13 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       if(ask_part_order!=0)
       {
 	/* Demande l'ordre des entrees dans le MBR */
-	ask_mbr_order(disk_car,list_part);
+	ask_mbr_order_i386(disk_car,list_part);
 	/* Demande l'ordre des partitions etendues */
       }
       do
       {
 	do_again=0;
-	res_interface_write=interface_write(disk_car,list_part,can_search_deeper,can_ask_minmax_ext,current_cmd,&menu);
+	res_interface_write=interface_write(disk_car,list_part,(fast_mode<1),can_ask_minmax_ext,current_cmd,&menu);
 	switch(res_interface_write)
 	{
 	  case 'W':
@@ -1044,7 +1181,6 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
 	    }
 	    break;
 	  case 'S':
-	    paranoid=1;
 	    if(fast_mode<2)
 	      fast_mode++;
 	    break;
