@@ -14,13 +14,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 enum part_offset { PART_OFFSET128, PART_OFFSET2, PART_OFFSET1, PART_OFFSET0, PART_QUIT};
 typedef enum part_offset part_offset_t;
-int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align,const int ask_part_order, const unsigned int expert, char **current_cmd);
+int interface_recovery(t_param_disk *disk_car, const int debug, const int dump_ind, const int align,const int ask_part_order, const unsigned int expert, char **current_cmd);
 void only_one_bootable( t_list_part *list_part, t_list_part *part_boot);
-t_list_part *search_part(t_param_disk *disk_car,const int paranoid,const int debug, const int dump_ind,const int fast_mode, const int interface);
+t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump_ind, const int fast_mode, const int interface);
