@@ -2,7 +2,7 @@
 
     File: intrface.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -24,6 +24,9 @@
 #endif
  
 #include <stdarg.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>	/* geteuid */
+#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -60,15 +63,14 @@
 #include "ntfs_dir.h"
 #include "rfs_dir.h"
 
-extern const t_arch_fnct arch_i386;
-extern const t_arch_fnct arch_sun;
-extern const t_arch_fnct arch_none;
-extern const t_arch_fnct arch_mac;
+#define INTER_DISK_X	0
+#define INTER_DISK_Y	7
 
-static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align, int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch, char**current_cmd);
+static void interface_options(int *dump_ind, int *align, int *allow_partial_last_cylinder, unsigned int *expert, char**current_cmd);
 static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part, const int debug);
 static t_list_part *backup_merge(t_list_part *list_part,t_list_part *backup_part, const int debug);
-static int write_MBR_code(t_param_disk *disk_car);
+static int write_MBR_code(t_param_disk *disk_car, char **current_cmd);
+static int write_clean_table(t_param_disk *disk_car, char **current_cmd);
 
 void interface_list(t_param_disk *disk_car, const int debug,const int test_recovery, const int saveheader, const int backup)
 {
@@ -100,8 +102,12 @@ void interface_list(t_param_disk *disk_car, const int debug,const int test_recov
   delete_list_part(list_part);
 }
 
-static int write_MBR_code(t_param_disk *disk_car)
+static int write_MBR_code(t_param_disk *disk_car, char **current_cmd)
 {
+  aff_copy(stdscr);
+  wmove(stdscr,5,0);
+  wdoprintf(stdscr,"%s\n",disk_car->description(disk_car));
+  wmove(stdscr,INTER_DISK_Y,INTER_DISK_X);
   if(disk_car->arch->write_MBR_code==NULL)
   {
     display_message("Function to write a new MBR code not implemented for this partition type.\n");
@@ -116,13 +122,17 @@ static int write_MBR_code(t_param_disk *disk_car)
       return 2;
     }
     else
-      display_message("You have to reboot for the change to take effect.\n");
+      display_message("A new copy of MBR code has been written.\nYou have to reboot for the change to take effect.\n");
   }
   return 0;
 }
 
-int write_clean_table(t_param_disk *disk_car)
+int write_clean_table(t_param_disk *disk_car, char**current_cmd)
 {
+  aff_copy(stdscr);
+  wmove(stdscr,5,0);
+  wdoprintf(stdscr,"%s\n",disk_car->description(disk_car));
+  wmove(stdscr,INTER_DISK_Y,INTER_DISK_X);
   if(disk_car->arch->erase_list_part==NULL)
   {
     display_message("Clear partition table not implemented for this partition type.\n");
@@ -137,37 +147,161 @@ int write_clean_table(t_param_disk *disk_car)
       return 2;
     }
     else
-      display_message("You have to reboot for the change to take effect.\n");
+      display_message("Partition table has been cleared.\nYou have to reboot for the change to take effect.\n");
   }
   return 0;
 }
 
-int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const t_arch_fnct **arch, const char *cmd_device, char *cmd_run)
+static int menu_disk(t_param_disk *disk_car, int debug,int dump_ind, const int saveheader, char **current_cmd)
 {
   int align=2;
   int allow_partial_last_cylinder=0;
   int ask_part_order=0;
   int command;
   int done=0;
-  int fast_mode=0;
-  int halt_on_errors=1;
   int menu=0;
-  int paranoid=0;
   int real_key;
   unsigned int expert=0;
+  char options[16];
+  static struct MenuItem menuMain[]=
+  {
+	{'A',"Analyse","Analyse current partition structure and search for lost partitions"},
+	{'T',"Advanced","Filesystem Utils"},
+	{'G',"Geometry", "Change disk geometry" },
+	{'O',"Options","Modify options"},
+	{'C',"MBR Code","Write TestDisk MBR code to first sector"},
+	{'D',"Delete","Delete all data in the partition table"},
+	{'Q',"Quit","Return to disk selection"},
+	{'E',"Editor","Basic disk editor"},
+	{0,NULL,NULL}
+  };
+  strcpy(options, "AGOPTQ");
+  if(disk_car->arch->write_MBR_code!=NULL)
+    strcat(options,"C");
+  if(disk_car->arch->erase_list_part!=NULL)
+    strcat(options,"D");
+  while(done==0)
+  {
+    aff_copy(stdscr);
+    wmove(stdscr,5,0);
+    wdoprintf(stdscr,"%s\n",disk_car->description(disk_car));
+    wmove(stdscr,20,0);
+    wdoprintf(stdscr,"Note: Correct disk geometry is required for a successful recovery. 'Analyse'");
+    wmove(stdscr,21,0);
+    wdoprintf(stdscr,"process may give some warnings if it thinks the logical geometry is mismatched.");
+    if(*current_cmd!=NULL)
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"analyze",7)==0 || strncmp(*current_cmd,"analyse",7)==0)
+      {
+	(*current_cmd)+=7;
+	command='A';
+      }
+      else if(strncmp(*current_cmd,"geometry",8)==0)
+      {
+	(*current_cmd)+=8;
+	command='G';
+      }
+      else if(strncmp(*current_cmd,"advanced",8)==0)
+      {
+	(*current_cmd)+=8;
+	command='T';
+      }
+      else if(strncmp(*current_cmd,"options",7)==0)
+      {
+	(*current_cmd)+=7;
+	command='O';
+      }
+      else if(strncmp(*current_cmd,"delete",6)==0)
+      {
+	(*current_cmd)+=6;
+	command='D';
+      }
+      else if(strncmp(*current_cmd,"mbr_code",8)==0)
+      {
+	(*current_cmd)+=8;
+	command='C';
+      }
+      else
+      {
+	if(*current_cmd[0]!='\0')
+	{
+	  ecrit_rapport("error >%s<\n",*current_cmd);
+	  while(*current_cmd[0]!='\0')
+	    (*current_cmd)++;
+	  return 1;
+	}
+	command='Q';
+      }
+    }
+    else
+      command = wmenuSelect_ext(stdscr,INTER_DISK_Y, INTER_DISK_X, menuMain, 10,
+	    options, MENU_VERT | MENU_VERT_WARN | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
+    /* e for editor will be added when the editor will be better */
+    switch(command)
+    {
+      case 'a':
+      case 'A':
+	interface_analyse(disk_car, debug, saveheader, current_cmd);
+	interface_recovery(disk_car, debug, dump_ind, align, ask_part_order, expert, current_cmd);
+	break;
+      case 'd':
+      case 'D':
+	write_clean_table(disk_car,current_cmd);
+	break;
+      case 'c':
+      case 'C':
+	write_MBR_code(disk_car,current_cmd);
+	break;
+      case 'g':
+      case 'G':
+	change_geometry(disk_car,current_cmd);
+	break;
+      case 'o':
+      case 'O':
+	{
+	  int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
+	  interface_options(&dump_ind, &align,&allow_partial_last_cylinder,&expert,current_cmd);
+	  if(old_allow_partial_last_cylinder!=allow_partial_last_cylinder)
+	    hd_update_geometry(disk_car,allow_partial_last_cylinder,debug);
+	}
+	break;
+      case 't':
+      case 'T':
+	interface_adv(disk_car, debug, dump_ind, expert,current_cmd);
+	break;
+      case 'q':
+      case 'Q':
+	done = 1;
+	break;
+      case 'e':
+      case 'E':
+	interface_editor(disk_car);
+	break;
+    }
+  }
+  return 0;
+}
+
+int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *real_prog_name, const char *cmd_device, char *cmd_run)
+{
+  int command;
+  int real_key;
+  int allow_partial_last_cylinder=0;
+  int done=0;
+  int menu=0;
+  int offset=0;
+  int pos_num=0;
   const t_list_disk *element_disk;
   const t_list_disk *current_disk=NULL;
   static struct MenuItem menuMain[]=
   {
-	{'A',"Analyse","Analyse current partition structure and search for lost partitions"},
-	{'T',"Advanced","Advanced boot recovery"},
-	{'G',"Geometry", "Change disk geometry (expert only)" },
-	{'O',"Options","Modify options"},
-	{'D',"Delete","Delete all data in the partition table"},
-	{'C',"MBR Code","Write the Classic MBR code to first sector"},
-	{'Q',"Quit","Quit program"},
-	{'E',"Editor","Basic disk editor"},
-	{0,NULL,NULL}
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'O',"Proceed",""},
+    { 'Q',"Quit","Quit program"},
+    { 0,NULL,NULL}
   };
   char *current_cmd=cmd_run;
   if(cmd_device!=NULL)
@@ -180,125 +314,95 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
   }
   else
     current_disk=list_disk;
-  if(current_disk==NULL)
-    return 0;
-  if(start_ncurses("TestDisk"))
+  if(start_ncurses("TestDisk",real_prog_name))
     return 1;
-  /* ncurses interface */
+  if(current_disk==NULL)
+  {
+    intrf_no_disk("TestDisk");
+    return 0;
+  }
+    /* ncurses interface */
   while(done==0)
   {
+    const char *options;
+    int i;
     aff_copy(stdscr);
+    wmove(stdscr,4,0);
+    wdoprintf(stdscr,"  TestDisk is free software, and");
     wmove(stdscr,5,0);
-    for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
+    wdoprintf(stdscr,"comes with ABSOLUTELY NO WARRANTY.");
+    wmove(stdscr,7,0);
+    wdoprintf(stdscr,"Select a media (use Arrow keys, then press Enter):");
+    for(i=0,element_disk=list_disk;(element_disk!=NULL) && (i<offset);element_disk=element_disk->next,i++);
+    for(;element_disk!=NULL && (i-offset)<10;i++,element_disk=element_disk->next)
     {
+      wmove(stdscr,8+i-offset,0);
       if(element_disk!=current_disk)
-	wdoprintf(stdscr,"%s\n",element_disk->disk->description(element_disk->disk));
+	wdoprintf(stdscr,"%s\n",element_disk->disk->description_short(element_disk->disk));
       else
       {
 	wattrset(stdscr,A_STANDOUT);
-	wdoprintf(stdscr,"%s\n",element_disk->disk->description(element_disk->disk));
+	wdoprintf(stdscr,"%s\n",element_disk->disk->description_short(element_disk->disk));
 	wattroff(stdscr,A_STANDOUT);
       }
     }
-    current_disk->disk->halt_on_errors=halt_on_errors;
+    if(i<=10 && element_disk==NULL)
+      options="OQ";
+    else
+      options="PNOQ";
+    wmove(stdscr,20,0);
+    wdoprintf(stdscr,"Note: Disk capacity must be correctly detected for a successful recovery.");
+    wmove(stdscr,21,0);
+    wdoprintf(stdscr,"If a disk listed above has incorrect size, check HD jumper settings, BIOS");
+    wmove(stdscr,22,0);
+    wdoprintf(stdscr,"detection, and install the latest OS patches and disk drivers."); 
     if(current_cmd!=NULL)
     {
       while(current_cmd[0]==',')
 	current_cmd++;
-      if(strncmp(current_cmd,"analyze",7)==0 || strncmp(current_cmd,"analyse",7)==0)
-      {
-	current_cmd+=7;
-	command='A';
-      }
-      else if(strncmp(current_cmd,"geometry",8)==0)
-      {
-	current_cmd+=8;
-	command='G';
-      }
-      else if(strncmp(current_cmd,"advanced",8)==0)
-      {
-	current_cmd+=8;
-	command='T';
-      }
-      else if(strncmp(current_cmd,"options",7)==0)
-      {
-	current_cmd+=7;
-	command='O';
-      }
-      else
-      {
-	if(current_cmd[0]!='\0')
-	  ecrit_rapport("error >%s<\n",current_cmd);
+      if(current_cmd[0]=='\0')
 	command='Q';
-      }
+      else
+	command='O';
     }
     else
       command = wmenuSelect_ext(stdscr,INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
-	    "AGDCOTQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
-    /* e for editor will be added when the editor will be better */
+	    options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
     switch(command)
     {
-      case 'a':
-      case 'A':
-	interface_analyse(current_disk->disk,debug,saveheader,&current_cmd);
-	interface_recovery(current_disk->disk,paranoid,debug, dump_ind, fast_mode,align,ask_part_order,expert,&current_cmd);
+      case KEY_UP:
+      case 'P':
+	if(current_disk->prev!=NULL)
+	{
+	  current_disk=current_disk->prev;
+	  pos_num--;
+	}
+	if(pos_num<offset)
+	  offset--;
 	break;
-      case 'd':
-      case 'D':
-	wmove(stdscr,INTER_MAIN_Y,INTER_MAIN_X);
-	write_clean_table(current_disk->disk);
-	break;
-      case 'c':
-      case 'C':
-	wmove(stdscr,INTER_MAIN_Y,INTER_MAIN_X);
-	write_MBR_code(current_disk->disk);
-	break;
-      case 'g':
-      case 'G':
-	change_geometry(current_disk->disk,&current_cmd);
+      case KEY_DOWN:
+      case 'N':
+	if(current_disk->next!=NULL)
+	{
+	  current_disk=current_disk->next;
+	  pos_num++;
+	}
+	if(pos_num>=offset+10)
+	  offset++;
 	break;
       case 'o':
       case 'O':
+	if(interface_partition_type(current_disk->disk, allow_partial_last_cylinder, debug, &current_cmd)==0)
 	{
-	  const t_arch_fnct *old_arch=*arch;
-	  int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
-	  interface_options(&paranoid,&dump_ind,&fast_mode,&align,&allow_partial_last_cylinder,&ask_part_order,&halt_on_errors,&expert,arch,&current_cmd);
-	  for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
-	  {
-	    element_disk->disk->arch=*arch;
-	  }
-	  if((old_allow_partial_last_cylinder!=allow_partial_last_cylinder) ||
-	      (old_arch!=*arch))
-	    hd_parse_bis(list_disk,allow_partial_last_cylinder,*arch,debug);
+	  if(menu_disk(current_disk->disk, debug, dump_ind, saveheader, &current_cmd))
+	    done=1;
+	  menu=3; /* select quit */
 	}
 	break;
-      case 't':
-      case 'T':
-	interface_adv(current_disk->disk, debug, dump_ind, expert,&current_cmd);
-	break;
       case 'q':
       case 'Q':
-	done = 1;
+	done=1;
 	break;
-      case 'e':
-      case 'E':
-	interface_editor(current_disk->disk);
-	break;
-      case KEY_UP:
-	if(current_disk->prev!=NULL)
-	  current_disk=current_disk->prev;
-	break;
-      case KEY_DOWN:
-	if(current_disk->next!=NULL)
-	  current_disk=current_disk->next;
-	break;
-#ifdef DEBUG
-      default:
-	wmove(stdscr,10,0);
-	wdoprintf(stdscr,"key %03X %d", command, command);
-	wrefresh(stdscr);
-	break;
-#endif
     }
   }
   end_ncurses();
@@ -310,13 +414,12 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
 {
   t_list_part *list_part;
   int command;
-  /* log */
   struct MenuItem menuAnalyse[]=
   {
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q',"Proceed","Try to locate partition"},
-    { 'T', "Save","Save partition structure"},
+    { 'S', "Save","Save partition structure"},
     { 0, NULL, NULL }
   };
   ecrit_rapport("\nAnalyse ");
@@ -328,12 +431,15 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
     mvwaddstr(stdscr,21,0,disk_car->arch->msg_part_type);
   wmove(stdscr,4,0);
   wdoprintf(stdscr,"%s\n",disk_car->description(disk_car));
-  mvwaddstr(stdscr,5,0,"Check current partition structure");
+  mvwaddstr(stdscr,5,0,"Checking current partition structure");
   wmove(stdscr,6,0);
   wrefresh(stdscr);
   wdoprintf(stdscr,msg_PART_HEADER_LONG);
   list_part=disk_car->arch->read_part(disk_car,debug,saveheader);
-  ecrit_rapport("Current partitions:\n");
+  wmove(stdscr,5,0);
+  wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
+  waddstr(stdscr,"Current partition structure:");
+  ecrit_rapport("Current partition structure:\n");
   if(*current_cmd!=NULL)
   {
     command='Q';
@@ -343,13 +449,13 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
     {
       (*current_cmd)+=6;
       if(list_part!=NULL)
-	command='T';
+	command='S';
     }
     screen_buffer_to_log();
   }
   else
-    command=screen_buffer_display(stdscr,(list_part!=NULL?"QT":"Q"),menuAnalyse);
-  if(command=='T')
+    command=screen_buffer_display(stdscr,(list_part!=NULL?"QS":"Q"),menuAnalyse);
+  if(command=='S')
   {
     ecrit_rapport("Backup partition structure\n");
     partition_save(disk_car,list_part,debug);
@@ -432,6 +538,9 @@ int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_
 	strcat(options,"S");
       if(disk_car->arch->write_part!=NULL)
 	strcat(options,"W");
+      else
+	aff_buffer(BUFFER_ADD," \nWrite isn't avaible because the partition table type \"%s\" has been selected.",
+	    disk_car->arch->part_name);
       if(can_ask_minmax_ext)
 	strcat(options,"E");
       command=screen_buffer_display_ext(stdscr,options,menuWrite,menu);
@@ -440,13 +549,11 @@ int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_
   return command;
 }
 
-static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align, int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch, char**current_cmd)
+static void interface_options(int *dump_ind, int *align, int *allow_partial_last_cylinder, unsigned int *expert, char**current_cmd)
 {
   /* paranoid: search in partition
    * dump_ind dump sector Y/N
-   * fast_mode Partitions are aligned on cylinder boundary Y/N
    * */
-
   if(*current_cmd!=NULL)
   {
     int keep_asking=1;
@@ -454,34 +561,13 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
     {
       while(*current_cmd[0]==',')
 	(*current_cmd)++;
-      if(strncmp(*current_cmd,"partition_i386",14)==0)
-      {
-	(*current_cmd)+=14;
-	*arch=&arch_i386;
-      }
-      else if(strncmp(*current_cmd,"partition_none",14)==0)
-      {
-	(*current_cmd)+=14;
-	*arch=&arch_none;
-      }
-      else if(strncmp(*current_cmd,"partition_sun",13)==0)
-      {
-	(*current_cmd)+=13;
-	*arch=&arch_sun;
-      }
-      else if(strncmp(*current_cmd,"partition_mac",13)==0)
-      {
-	(*current_cmd)+=13;
-	*arch=&arch_mac;
-      }
-      else
-	keep_asking=0;
+      keep_asking=0;
     } while(keep_asking>0);
   }
   else
   {
     int done = FALSE;
-    int menu = 9;
+    int menu = 4;
     /* ncurses interface */
     while (done==FALSE)
     {
@@ -489,58 +575,33 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
       int real_key;
       struct MenuItem menuOptions[]=
       {
-	{ 'P', NULL, "Search all partitions" },
-	{ 'D',NULL,"Dump essential sectors" },
-	{ 'C',NULL,"Partitions are aligned on cylinder/head boundary" },
-	{ 'F',NULL,"Don't check every hard disk head" },
+	{ 'E',NULL,"Expert mode adds some functionaliteis"},
+	{ 'C',NULL,"Partitions are aligned on cylinder/head boundaries" },
 	{ 'A',NULL,""},
-	{ 'O',NULL,""},
-	{ 'H',NULL,"Halt on Disk I/O errors" },
-	{ 'E',NULL,"Expert mode"},
-	{ 'T',NULL,"Partition table type"},
-	{ 'Q',"[Ok]","Return to main menu"},
+	{ 'D',NULL,"Dump essential sectors" },
+	{ 'Q',"[ Ok ]","Done with changing options"},
 	{ 0, NULL, NULL }
       };
-      menuOptions[0].name=*paranoid?"Paranoid : Yes":"Paranoid : No";
-      menuOptions[1].name=*dump_ind?"Dump : Yes":"Dump : No";
+      menuOptions[0].name=*expert?"Expert mode : Yes":"Expert mode : No";
       switch(*align)
       {
 	case 0:
-	  menuOptions[2].name="Cylinder boundary : No";
+	  menuOptions[1].name="Cylinder boundary : No";
 	  break;
 	case 1:
-	  menuOptions[2].name="Cylinder boundary : Head boundary only";
+	  menuOptions[1].name="Cylinder boundary : Head boundary only";
 	  break;
 	case 2:
-	  menuOptions[2].name="Cylinder boundary : Yes";
+	  menuOptions[1].name="Cylinder boundary : Yes";
 	  break;
       }
-      switch(*fast_mode)
-      {
-	case 0:
-	  menuOptions[3].name="Mode : Fast";
-	  break;
-	case 1:
-	  menuOptions[3].name="Mode : Medium";
-	  break;
-	default:
-	  menuOptions[3].name="Mode : Slow";
-	  break;
-      }
-      menuOptions[4].name=*allow_partial_last_cylinder?"Allow partial last cylinder : Yes":"Allow partial last cylinder : No";
-      menuOptions[5].name=*ask_part_order?"Ask partition order : Yes":"Ask partition order : No";
-      menuOptions[6].name=*halt_on_errors?"Halt on errors : Yes":"Halt on errors : No";
-      menuOptions[7].name=*expert?"Expert mode : Yes":"Expert mode : No";
-      menuOptions[8].name=(*arch)->part_name;
+      menuOptions[2].name=*allow_partial_last_cylinder?"Allow partial last cylinder : Yes":"Allow partial last cylinder : No";
+      menuOptions[3].name=*dump_ind?"Dump : Yes":"Dump : No";
       aff_copy(stdscr);
 
-      car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "DCAOHMETQ", MENU_VERT, &menu,&real_key);
+      car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "ECADQ", MENU_VERT, &menu,&real_key);
       switch(car)
       {
-	case 'p':
-	case 'P':
-	  *paranoid=!*paranoid;
-	  break;
 	case 'd':
 	case 'D':
 	  *dump_ind=!*dump_ind;
@@ -552,54 +613,14 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
 	  else
 	    *align=0;
 	  break;
-	case 'f':
-	case 'F':
-	  if(*fast_mode<2)
-	    (*fast_mode)++;
-	  else
-	    *fast_mode=0;
-	  break;
 	case 'a':
 	case 'A':
 	  *allow_partial_last_cylinder=!*allow_partial_last_cylinder;
 	  break;
-	case 'o':
-	case 'O':
-	  *ask_part_order=!*ask_part_order;
-	  break;
-	case 'h':
-	case 'H':
-	  *halt_on_errors=!*halt_on_errors;
-	  break;
 	case 'e':
 	case 'E':
 	  *expert=!*expert;
 	  break;
-	case 't':
-	case 'T':
-	  if(real_key==KEY_LEFT)
-	  {
-	    if(*arch==&arch_i386)
-	      *arch=&arch_none;
-	    else if(*arch==&arch_sun)
-	      *arch=&arch_i386;
-	    else if(*arch==&arch_mac)
-	      *arch=&arch_sun;
-	    else
-	      *arch=&arch_mac;
-	  }
-	  else
-	  {
-	    if(*arch==&arch_i386)
-	      *arch=&arch_sun;
-	    else if(*arch==&arch_sun)
-	      *arch=&arch_mac;
-	    else if(*arch==&arch_mac)
-	      *arch=&arch_none;
-	    else
-	      *arch=&arch_i386;
-	  }
-	  break;
 	case key_ESC:
 	case 'q':
 	case 'Q':
@@ -609,8 +630,7 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
     }
   }
   /* write new options to log file */
-  ecrit_rapport("New options :\n Paranoid : %s\n Dump : %s\n ",
-      *paranoid?"Yes":"No",
+  ecrit_rapport("New options :\n Dump : %s\n ",
       *dump_ind?"Yes":"No");
   switch(*align)
   {
@@ -624,25 +644,9 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
       ecrit_rapport("Cylinder boundary : Yes");
       break;
   }
-  ecrit_rapport("\n ");
-  switch(*fast_mode)
-  {
-    case 0:
-      ecrit_rapport("Mode : Fast");
-      break;
-    case 1:
-      ecrit_rapport("Mode : Medium");
-      break;
-    default:
-      ecrit_rapport("Mode : Slow");
-      break;
-  }
-  ecrit_rapport("\n Allow partial last cylinder : %s\n Ask partition order : %s\n Halt on errors : %s\n Expert mode : %s\n %s\n",
+  ecrit_rapport("\n Allow partial last cylinder : %s\n Expert mode : %s\n",
       *allow_partial_last_cylinder?"Yes":"No",
-      *ask_part_order?"Yes":"No",
-      *halt_on_errors?"Yes":"No",
-      *expert?"Yes":"No",
-      (*arch)->part_name);
+      *expert?"Yes":"No");
 }
 
 t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd)
@@ -676,7 +680,6 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       {
 	wattrset(stdscr,A_STANDOUT);
       }
-      /* TODO status letter in green unless D(eleted) or structure_status!=0 */
       if(structure_status==0 && parts->part->status!=STATUS_DELETED && has_colors())
 	wbkgdset(stdscr,' ' | COLOR_PAIR(2));
       aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
@@ -684,6 +687,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
       if(parts==pos)
       {
+	char buffer_part_size[100];
 	wattroff(stdscr,A_STANDOUT);
 	wmove(stdscr,24,0);
 	wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
@@ -691,36 +695,79 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	{
 	  wdoprintf(stdscr,"%s, ",parts->part->info);
 	}
-	wdoprintf(stdscr,"%lu MB",(long unsigned)(parts->part->part_size/(1024*1024)));
+	wdoprintf(stdscr,"%s",size_to_unit(parts->part->part_size,buffer_part_size));
       }
     }
     if(structure_status==0)
-      mvwaddstr(stdscr,20,0,msg_STRUCT_OK);
+      mvwaddstr(stdscr,19,0,msg_STRUCT_OK);
     else
     {
       if(has_colors())
-	wbkgdset(stdscr,' ' | COLOR_PAIR(1));
-      mvwaddstr(stdscr,20,0,msg_STRUCT_BAD);
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(1));
+      mvwaddstr(stdscr,19,0,msg_STRUCT_BAD);
       if(has_colors())
-	bkgdset(' ' | COLOR_PAIR(0));
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
     }
     if(list_part!=NULL && disk_car->arch->msg_part_type!=NULL)
     {
-      mvwaddstr(stdscr,20,16,"Use arrow keys to change partition characteristics:");
+      mvwaddstr(stdscr,19,16,"Use ");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+      waddstr(stdscr,"Up");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      waddstr(stdscr,"/");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+      waddstr(stdscr,"Down");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      waddstr(stdscr," Arrow keys to select partition.");
+      mvwaddstr(stdscr,20,0,"Use ");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+      waddstr(stdscr,"Left");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      waddstr(stdscr,"/");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+      waddstr(stdscr,"Right");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      waddstr(stdscr," Arrow keys to CHANGE partition characteristics:");
       mvwaddstr(stdscr,21,0,disk_car->arch->msg_part_type);
     }
     wmove(stdscr,22,0);
     wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
-    waddstr(stdscr,"Keys A: add partition, L: load backup, ");
+    waddstr(stdscr,"Keys ");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+    waddstr(stdscr,"A");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+    waddstr(stdscr,": add partition, ");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+    waddstr(stdscr,"L");
+    if(has_colors())
+      wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+    waddstr(stdscr,": load backup, ");
+
     if(list_part==NULL)
     {
-      waddstr(stdscr,"ENTER: to continue");
+      waddstr(stdscr,"Enter: to continue");
     }
     else
     {
       if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
       {
-	waddstr(stdscr,"T: change type, ");
+	if(has_colors())
+	  wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+	waddstr(stdscr,"T");
+	if(has_colors())
+	  wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+	waddstr(stdscr,": change type, ");
       }
       switch(pos->part->upart_type)
       {
@@ -733,12 +780,22 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	case UP_FAT16:
 	case UP_FAT32:
 	case UP_NTFS:
-	  waddstr(stdscr,"P: list files, ");
+	  if(has_colors())
+	    wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+	  waddstr(stdscr,"P");
+	  if(has_colors())
+	    wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+	  waddstr(stdscr,": list files, ");
 	  break;
 	default:
 	  break;
       }
-      mvwaddstr(stdscr,23,5, "ENTER: to continue");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | A_BOLD | COLOR_PAIR(0));
+      mvwaddstr(stdscr,23,5, "Enter");
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      waddstr(stdscr,": to continue");
     }
     wrefresh(stdscr);
     if(*current_cmd!=NULL)
@@ -839,7 +896,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       case 'a':
       case 'A':
 	{
-	  list_part=disk_car->arch->add_partition(disk_car,list_part, debug);
+	  list_part=disk_car->arch->add_partition(disk_car,list_part, debug,current_cmd);
 	  rewrite=1;
 	  offset=0;
 	  pos_num=0;
@@ -853,7 +910,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	  if(pos->part->arch==NULL || pos->part->arch==disk_car->arch)
 	  {
 	    rewrite=1;
-	    change_part_type(disk_car,pos->part);
+	    change_part_type(disk_car,pos->part,current_cmd);
 	  }
 	}
 	break;
@@ -946,7 +1003,7 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
     {
       { 'P', "Previous",""},
       { 'N', "Next","" },
-      { 'L',"Load","Load partition structure from backup and try to location partition"},
+      { 'L',"Load","Load partition structure from backup and try to locate partition"},
       { 'Q',"Cancel","Don't use backup and try to locate partition"},
       { 0, NULL, NULL }
     };
@@ -1129,3 +1186,4 @@ int interface_superblock(t_param_disk *disk_car,t_list_part *list_part, char**cu
   }
   return screen_buffer_display(stdscr,"",menuSuperblock);
 }
+
