@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -64,7 +64,6 @@ int io_redir_add_redir(t_param_disk *disk_car, const uint64_t org_offset, const
     memcpy(old_disk_car,disk_car,sizeof(*old_disk_car));
     data->disk_car=old_disk_car;
     data->list_redir=NULL;
-    disk_car->halt_on_errors=0;
     disk_car->write_used=0;
     disk_car->data=data;
     disk_car->description=old_disk_car->description;
