@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -70,16 +70,15 @@ int set_MD_info(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_part
 
 int recover_MD(t_param_disk *disk_car, const struct mdp_superblock_s *sb,t_partition *partition,const int debug, const int dump_ind)
 {
+  uint64_t offset;
   if(test_MD(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
   set_MD_info(disk_car,sb,partition,debug,dump_ind);
   partition->part_type_i386=P_RAID;
   partition->part_type_sun=PSUN_RAID;
   partition->part_size=(uint64_t)(sb->size<<1)*disk_car->sector_size+MD_RESERVED_BYTES;
-  partition->part_offset=partition->part_offset-partition->part_size;
-  /*
-  partition->part_offset=partition->part_offset-(partition->part_size+sb->chunk_size-1)/sb->chunk_size*sb->chunk_size;
-  */
+  offset=MD_NEW_SIZE_SECTORS(partition->part_size/512)*512;
+  partition->part_offset=partition->part_offset-offset;
   return 0;
 }
 
