@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -30,4 +30,4 @@ typedef struct disk_netware
 } disk_netware;
 
 int check_netware(t_param_disk *disk_car,t_partition *partition,const int debug);
-int recover_netware(t_param_disk *disk_car, const struct disk_netware *,t_partition *);
+int recover_netware(t_param_disk *disk_car, const struct disk_netware *netware_block, t_partition *partition);
