@@ -2,7 +2,7 @@
 
     File: ntfs.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 /*define NTFS_DEBUG 1 */
@@ -38,9 +38,7 @@
 #include "lang.h"
 
 static int ntfs_read_MFT(t_param_disk *disk_car, t_partition *partition, const struct ntfs_boot_sector*ntfs_header,const int my_type,const int debug, const int dump_ind);
-
 static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
-unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header);
 
 unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header)
 { return (ntfs_header->sector_size[1]<<8)+ntfs_header->sector_size[0]; }
@@ -96,11 +94,18 @@ int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_he
   {
     if(memcmp(ntfs_header->system_id,"NTFS",4)==0)
     {
-      if((ntfs_header->reserved>0) || (ntfs_header->fats>0) ||
+      if((le16(ntfs_header->reserved)>0) || (ntfs_header->fats>0) ||
 	  (ntfs_header->dir_entries[0]!=0) || (ntfs_header->dir_entries[1]!=0) ||
 	  (ntfs_header->sectors[0]!=0) || (ntfs_header->sectors[1]!=0) ||
-	  (ntfs_header->fat_length!=0) || (ntfs_header->total_sect!=0) || (ntfs_header->sectors_per_cluster==0))
+	  (le16(ntfs_header->fat_length)!=0) || (le32(ntfs_header->total_sect)!=0))
 	return 1;
+      switch(ntfs_header->sectors_per_cluster)
+      {
+	case 1: case 2: case 4: case 8: case 16: case 32: case 64: case 128:
+	  break;
+	default:
+	  return 1;
+      }
       if((debug>0) || (dump_ind!=0))
       {
 	ecrit_rapport("NTFS at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
@@ -111,7 +116,6 @@ int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_he
       {
 	aff_buffer(BUFFER_ADD,"test_NTFS: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
 	ecrit_rapport("test_NTFS: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",ntfs_sector_size(ntfs_header),disk_car->sector_size);
-	return 1;
       }
       if(le16(ntfs_header->secs_track)!=disk_car->CHS.sector)
       {
@@ -186,7 +190,10 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_parti
     attr_len=NTFS_GETU16(attr_record+4);
     if((attr_len%8!=0)||(attr_len<0x18))
     {
-      ecrit_rapport("ntfs_get_attr attr_type=%x attr_len=%u (attr_len%%8!0)||(attr_len<0x18)\n",attr_type,attr_len);
+      if(debug>1)
+      {
+	ecrit_rapport("ntfs_get_attr attr_type=%x attr_len=%u (attr_len%%8!0)||(attr_len<0x18)\n",attr_type,attr_len);
+      }
       return 2;
     }
     if(debug>1)
@@ -242,7 +249,7 @@ static int ntfs_get_attr_aux(const char *attr_record, const int my_type, t_parti
 	      if(file_name_attr+0x42+2*file_name_length>=end)
 		return 2;
 	      {
-		char file_name[file_name_length+1];
+		char file_name[256+1];	/* used size is file_name_length+1 */
 		unsigned int i;
 		/*		aff_buffer(BUFFER_ADD,"MFT record nbr %lu ",NTFS_GETU64(file_name_attr)); */
 		for(name_it=file_name_attr+0x42,i=0;i<file_name_length; name_it+=2,i++)
@@ -369,27 +376,27 @@ static int ntfs_read_MFT(t_param_disk *disk_car, t_partition *partition, const s
   char *attr;
   uint64_t mft_pos;
   unsigned int mft_recordsize;
-  mft_pos=partition->part_offset+(uint64_t)(ntfs_header->reserved+ntfs_header->mft_lcn*ntfs_header->sectors_per_cluster)*DEFAULT_SECTOR_SIZE;
+  mft_pos=partition->part_offset+(uint64_t)(le16(ntfs_header->reserved)+le64(ntfs_header->mft_lcn)*ntfs_header->sectors_per_cluster)*ntfs_sector_size(ntfs_header);
   if(ntfs_header->clusters_per_index_record>0)
     mft_recordsize=ntfs_header->sectors_per_cluster*ntfs_header->clusters_per_index_record;
   else
     mft_recordsize=1<<(-ntfs_header->clusters_per_index_record);
-  buffer=(unsigned char *)MALLOC(mft_recordsize*DEFAULT_SECTOR_SIZE);
+  buffer=(unsigned char *)MALLOC(mft_recordsize*ntfs_sector_size(ntfs_header));
 #ifdef NTFS_DEBUG
   ecrit_rapport("NTFS cluster size = %u\n",ntfs_header->sectors_per_cluster);
   ecrit_rapport("NTFS MFT_record_size = %u\n",mft_recordsize);
-  ecrit_rapport("NTFS MFT cluster = %lu\n",ntfs_header->mft_lcn);
+  ecrit_rapport("NTFS MFT cluster = %lu\n",le64(ntfs_header->mft_lcn));
 #endif
-  if(disk_car->read(disk_car,mft_recordsize*DEFAULT_SECTOR_SIZE, buffer, mft_pos)!=0)
+  if(disk_car->read(disk_car,mft_recordsize*ntfs_sector_size(ntfs_header), buffer, mft_pos)!=0)
   {
     ecrit_rapport("NTFS: Can't read MFT\n");
     FREE(buffer);
     return 1;
   }
   attr=(char*)buffer;
-  while(attr+0x30<=(char*)(buffer+mft_recordsize*DEFAULT_SECTOR_SIZE))
+  while(attr+0x30<=(char*)(buffer+mft_recordsize*ntfs_sector_size(ntfs_header)))
   {
-    int res=ntfs_get_attr(attr,my_type,partition,(char*)buffer+mft_recordsize*DEFAULT_SECTOR_SIZE,debug,dump_ind,NULL);
+    int res=ntfs_get_attr(attr,my_type,partition,(char*)buffer+mft_recordsize*ntfs_sector_size(ntfs_header),debug,dump_ind,NULL);
     if((res>0)|| (NTFS_GETU32(attr + 0x1C)<0x30))
     {
       FREE(buffer);
@@ -416,10 +423,11 @@ int is_ntfs(const t_partition *partition)
 
 int dump_ntfs_rapport(const struct ntfs_boot_sector *ntfs_header)
 {
-  ecrit_rapport("sectors_per_cluster       %u\n",ntfs_header->sectors_per_cluster);
-  ecrit_rapport("mft_lcn                   %lu\n",(long unsigned int)ntfs_header->mft_lcn);
-  ecrit_rapport("mftmirr_lcn               %lu\n",(long unsigned int)ntfs_header->mftmirr_lcn);
-  ecrit_rapport("clusters_per_mft_record   %d\n",ntfs_header->clusters_per_mft_record);
-  ecrit_rapport("clusters_per_index_record %d\n",ntfs_header->clusters_per_index_record);
+  ecrit_rapport("filesystem size           %llu\n", (long long unsigned)le64(ntfs_header->sectors_nbr)+1);
+  ecrit_rapport("sectors_per_cluster       %u\n", ntfs_header->sectors_per_cluster);
+  ecrit_rapport("mft_lcn                   %lu\n", (long unsigned int)le64(ntfs_header->mft_lcn));
+  ecrit_rapport("mftmirr_lcn               %lu\n", (long unsigned int)le64(ntfs_header->mftmirr_lcn));
+  ecrit_rapport("clusters_per_mft_record   %d\n", ntfs_header->clusters_per_mft_record);
+  ecrit_rapport("clusters_per_index_record %d\n", ntfs_header->clusters_per_index_record);
   return 0;
 }
