@@ -38,7 +38,10 @@ int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_head
 int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind);
 int test_NTFS(const t_param_disk *disk_car,const struct ntfs_boot_sector*ntfs_header, t_partition *partition,const int debug, const int dump_ind);
 #define NTFS_GETU8(p)      (*(const uint8_t*)(p))
-#define NTFS_GETU16(p)     (*(const uint16_t*)(p))
-#define NTFS_GETU32(p)     (*(const uint32_t*)(p))
-#define NTFS_GETU64(p)     (*(const uint64_t*)(p))
+#define NTFS_GETU16(p)     (le16(*(const uint16_t*)(p)))
+#define NTFS_GETU32(p)     (le32(*(const uint32_t*)(p)))
+#define NTFS_GETU64(p)     (le64(*(const uint64_t*)(p)))
+unsigned int ntfs_sector_size(const struct ntfs_boot_sector *ntfs_header);
+int rebuild_NTFS_BS(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind,const int interface, const unsigned int expert, char**current_cmd);
+int repair_MFT(t_param_disk *disk_car, t_partition *partition, const int debug);
  
