@@ -2,7 +2,7 @@
 
     File: ntfs_adv.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -76,17 +76,15 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
   ntfs_header->sectors[0]=0;
   ntfs_header->sectors[1]=0;
   ntfs_header->media=0xF8;
-  ntfs_header->fat_length=0;
+  ntfs_header->fat_length=le16(0);
   ntfs_header->secs_track=le16(disk_car->CHS.sector);
   ntfs_header->heads=le16(disk_car->CHS.head+1);
-  if(partition->status==STATUS_LOG)
-    ntfs_header->hidden=63;
-  else
-    ntfs_header->hidden=partition->part_offset/disk_car->sector_size;
-  ntfs_header->total_sect=0;
+  /* absolute sector address from the beginning of the disk (!= FAT) */
+  ntfs_header->hidden=le32(partition->part_offset/disk_car->sector_size);
+  ntfs_header->total_sect=le32(0);
   ntfs_header->sectors_nbr=le64(partition->part_size/disk_car->sector_size-1);
-  ntfs_header->mft_lcn=mft_lcn;
-  ntfs_header->mftmirr_lcn=mftmirr_lcn;
+  ntfs_header->mft_lcn=le64(mft_lcn);
+  ntfs_header->mftmirr_lcn=le64(mftmirr_lcn);
   ntfs_header->clusters_per_mft_record=(mft_record_size >= cluster_size?mft_record_size / cluster_size:
       -(testdisk_ffs(mft_record_size) - 1));
   ntfs_header->clusters_per_index_record =(index_block_size >= cluster_size?index_block_size / cluster_size:
@@ -168,6 +166,11 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
 	  command='L';
 	  (*current_cmd)+=4;
 	}
+	else if(strncmp(*current_cmd,"dump",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='D';
+	}
       }
       else
 	command=wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuSaveBoot,8,options,MENU_HORIZ | MENU_BUTTON, 1);
@@ -182,7 +185,21 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
 	case 'D':
 	  if(strchr(options,'D')!=NULL)
 	  {
-	    dump2(stdscr,newboot,orgboot, sizeof(newboot));
+	    WINDOW *window=newwin(0,0,0,0);	/* full screen */
+	    keypad(window, TRUE); /* Need it to get arrow key */
+	    aff_copy(window);
+	    wmove(window,4,0);
+	    wdoprintf(window,"%s",disk_car->description(disk_car));
+	    wmove(window,5,0);
+	    aff_part(window,AFF_PART_ORDER,disk_car,partition);
+	    ecrit_rapport("     Rebuild Boot sector           Boot sector\n");
+	    mvwaddstr(window,6,0, "     Rebuild Boot sector           Boot sector");
+	    dump2(window,newboot,orgboot, sizeof(newboot));
+	    delwin(window);
+	    (void) clearok(stdscr, TRUE);
+#ifdef HAVE_TOUCHWIN
+	    touchwin(stdscr);
+#endif
 	  }
 	  break;
 	case 'l':
@@ -209,11 +226,11 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
       {
 	display_message("Write error: Can't write new NTFS backup boot sector\n");
       }
+      return 0;
     }
-    else
-      ecrit_rapport("Don't write new boot!\n");
   }
-  return 0;
+  ecrit_rapport("Don't write new NTFS boot sector and backup boot sector!\n");
+  return 1;
 }
 
 static int read_mft_info(t_param_disk *disk_car, t_partition *partition, uint64_t mft_sector, const int debug, unsigned int *sectors_per_cluster, uint64_t *mft_lcn, uint64_t *mftmirr_lcn, unsigned int *mft_record_size)
@@ -241,12 +258,24 @@ static int read_mft_info(t_param_disk *disk_car, t_partition *partition, uint64_
     if(*mftmirr_lcn>0 && mft_sector%(*mftmirr_lcn)==0)
     {
       *sectors_per_cluster=mft_sector/(*mftmirr_lcn);
-      return 0;
+      switch(*sectors_per_cluster)
+      {
+	case 1: case 2: case 4: case 8: case 16: case 32: case 64: case 128:
+	  return 0;
+	default:
+	  break;
+      }
     }
     if(*mft_lcn>0 && mft_sector%(*mft_lcn)==0)
     {
       *sectors_per_cluster=mft_sector/(*mft_lcn);
-      return 0;
+      switch(*sectors_per_cluster)
+      {
+	case 1: case 2: case 4: case 8: case 16: case 32: case 64: case 128:
+	  return 0;
+	default:
+	  break;
+      }
     }
   }
   else
@@ -254,18 +283,30 @@ static int read_mft_info(t_param_disk *disk_car, t_partition *partition, uint64_
     if(*mft_lcn>0 && mft_sector%(*mft_lcn)==0)
     {
       *sectors_per_cluster=mft_sector/(*mft_lcn);
-      return 0;
+      switch(*sectors_per_cluster)
+      {
+	case 1: case 2: case 4: case 8: case 16: case 32: case 64: case 128:
+	  return 0;
+	default:
+	  break;
+      }
     }
     if(*mftmirr_lcn>0 && mft_sector%(*mftmirr_lcn)==0)
     {
       *sectors_per_cluster=mft_sector/(*mftmirr_lcn);
-      return 0;
+      switch(*sectors_per_cluster)
+      {
+	case 1: case 2: case 4: case 8: case 16: case 32: case 64: case 128:
+	  return 0;
+	default:
+	  break;
+      }
     }
   }
   if(debug>0)
   {
     ecrit_rapport("read_mft_info failed\n");
-    ecrit_rapport("ntfs_find_mft: sectors_per_cluster %u\n",*sectors_per_cluster);
+    ecrit_rapport("ntfs_find_mft: sectors_per_cluster invalid\n");
     ecrit_rapport("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)*mft_lcn);
     ecrit_rapport("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)*mftmirr_lcn);
     ecrit_rapport("ntfs_find_mft: mft_record_size     %u\n",*mft_record_size);
@@ -274,7 +315,7 @@ static int read_mft_info(t_param_disk *disk_car, t_partition *partition, uint64_
   return 3;
 }
 
-int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface, char **current_cmd)
+int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface, const unsigned int expert, char **current_cmd)
 {
   uint64_t sector;
   char buffer[8*DEFAULT_SECTOR_SIZE];
@@ -322,9 +363,15 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
     {
       wmove(stdscr,9,0);
       wclrtoeol(stdscr);
-      wdoprintf(stdscr,"Search mft %10lu/%lu",(long unsigned)sector,(long unsigned)(partition->part_size/disk_car->sector_size));
+      wdoprintf(stdscr,"Search mft %10lu/%lu", (long unsigned)sector,
+	  (long unsigned)(partition->part_size/disk_car->sector_size));
       wrefresh(stdscr);
-      ind_stop|=check_enter_or_s(stdscr);
+      if(check_enter_or_s(stdscr))
+      {
+	ecrit_rapport("Search mft stopped: %10lu/%lu)\n", (long unsigned)sector,
+	    (long unsigned)(partition->part_size/disk_car->sector_size));
+	ind_stop=TRUE;
+      }
     }
     if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, &buffer, partition->part_offset+sector*(uint64_t)disk_car->sector_size)==0)
     {
@@ -345,6 +392,13 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
       }
     }
   }
+  if(interface>0 && expert>0)
+  {
+    sectors_per_cluster=ask_number(sectors_per_cluster,0,512,"Sectors per cluster ");
+    mft_lcn=ask_number(mft_lcn,0,0,"MFT LCN ");
+    mftmirr_lcn=ask_number(mftmirr_lcn,0,0,"MFTMIRR LCN ");
+    mft_record_size=ask_number(mft_record_size,0,4096," mft record size ");
+  }
   /* TODO read_mft_info(partition,sector,*sectors_per_cluster,*mft_lcn,*mftmirr_lcn,*mft_record_size); */
   if(sectors_per_cluster>0 && mft_record_size>0)
   {
@@ -364,6 +418,7 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
     ecrit_rapport("ntfs_find_mft: mft_record_size     %u\n",mft_record_size);
     ecrit_rapport("ntfs_find_mft: index_block_size    %u\n",index_block_size);
     create_ntfs_boot_sector(disk_car,partition, interface, sectors_per_cluster*disk_car->sector_size, mft_lcn, mftmirr_lcn, mft_record_size, index_block_size,current_cmd);
+    /* TODO: ask if the user want to continue the search of MFT */
   }
   else
   {
@@ -374,6 +429,7 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
 
 static int dump_ntfs_info(const struct ntfs_boot_sector *ntfs_header)
 {
+  wdoprintf(stdscr,"filesystem size           %llu\n", (long long unsigned)(le64(ntfs_header->sectors_nbr)+1));
   wdoprintf(stdscr,"sectors_per_cluster       %u\n",ntfs_header->sectors_per_cluster);
   wdoprintf(stdscr,"mft_lcn                   %lu\n",(long unsigned int)ntfs_header->mft_lcn);
   wdoprintf(stdscr,"mftmirr_lcn               %lu\n",(long unsigned int)ntfs_header->mftmirr_lcn);
@@ -413,9 +469,12 @@ static int testdisk_ffs(int x)
 
 static int dump_2ntfs_info(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2)
 {
+  wdoprintf(stdscr,"filesystem size           %llu %llu\n",
+      (long long unsigned)(le64(nh1->sectors_nbr)+1),
+      (long long unsigned)(le64(nh2->sectors_nbr)+1));
   wdoprintf(stdscr,"sectors_per_cluster       %u %u\n",nh1->sectors_per_cluster,nh2->sectors_per_cluster);
-  wdoprintf(stdscr,"mft_lcn                   %lu %lu\n",(long unsigned int)nh1->mft_lcn,(long unsigned int)nh2->mft_lcn);
-  wdoprintf(stdscr,"mftmirr_lcn               %lu %lu\n",(long unsigned int)nh1->mftmirr_lcn,(long unsigned int)nh2->mftmirr_lcn);
+  wdoprintf(stdscr,"mft_lcn                   %lu %lu\n",(long unsigned int)le64(nh1->mft_lcn),(long unsigned int)le64(nh2->mft_lcn));
+  wdoprintf(stdscr,"mftmirr_lcn               %lu %lu\n",(long unsigned int)nh1->mftmirr_lcn,(long unsigned int)le64(nh2->mftmirr_lcn));
   wdoprintf(stdscr,"clusters_per_mft_record   %d %d\n",nh1->clusters_per_mft_record,nh2->clusters_per_mft_record);
   wdoprintf(stdscr,"clusters_per_index_record %d %d\n",nh1->clusters_per_index_record,nh2->clusters_per_index_record);
   return 0;
@@ -423,9 +482,12 @@ static int dump_2ntfs_info(const struct ntfs_boot_sector *nh1, const struct ntfs
 
 int dump_2ntfs_rapport(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2)
 {
+  ecrit_rapport("filesystem size           %llu %llu\n",
+      (long long unsigned)(le64(nh1->sectors_nbr)+1),
+      (long long unsigned)(le64(nh2->sectors_nbr)+1));
   ecrit_rapport("sectors_per_cluster       %u %u\n",nh1->sectors_per_cluster,nh2->sectors_per_cluster);
-  ecrit_rapport("mft_lcn                   %lu %lu\n",(long unsigned int)nh1->mft_lcn,(long unsigned int)nh2->mft_lcn);
-  ecrit_rapport("mftmirr_lcn               %lu %lu\n",(long unsigned int)nh1->mftmirr_lcn,(long unsigned int)nh2->mftmirr_lcn);
+  ecrit_rapport("mft_lcn                   %lu %lu\n",(long unsigned int)le64(nh1->mft_lcn),(long unsigned int)le64(nh2->mft_lcn));
+  ecrit_rapport("mftmirr_lcn               %lu %lu\n",(long unsigned int)le64(nh1->mftmirr_lcn),(long unsigned int)le64(nh2->mftmirr_lcn));
   ecrit_rapport("clusters_per_mft_record   %d %d\n",nh1->clusters_per_mft_record,nh2->clusters_per_mft_record);
   ecrit_rapport("clusters_per_index_record %d %d\n",nh1->clusters_per_index_record,nh2->clusters_per_index_record);
   return 0;
