@@ -47,6 +47,7 @@
 #ifdef HAVE_MACHINE_ENDIAN_H
 #include <machine/endian.h>
 #endif
+#include <stdarg.h>
 #include <ntfs/attrib.h>
 #ifdef HAVE_NTFS_VERSION_H
 #include <ntfs/version.h>
@@ -85,12 +86,15 @@ extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
                 void *dirent, ntfs_filldir_t filldir);
 extern u64 ntfs_inode_lookup_by_name(ntfs_inode *dir_ni,
                 const ntfschar *uname, const int uname_len);
-time_t ntfs2utc (s64 ntfstime);
-int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
+static time_t ntfs2utc (s64 ntfstime);
+static int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
-ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname);
-ATTR_RECORD * find_attribute (const ATTR_TYPES type, ntfs_attr_search_ctx *ctx);
+static ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname);
+static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
+static ATTR_RECORD * find_attribute(const ATTR_TYPES type, ntfs_attr_search_ctx *ctx);
+static ATTR_RECORD * find_first_attribute(const ATTR_TYPES type, MFT_RECORD *mft);
+static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_inode);
 
 /**
  * ntfs2utc - Convert an NTFS time to Unix time
@@ -101,7 +105,7 @@ ATTR_RECORD * find_attribute (const ATTR_TYPES type, ntfs_attr_search_ctx *ctx);
  *
  * Return:  n  A Unix time (number of seconds since 1970)
  */
-time_t ntfs2utc (s64 ntfstime)
+static time_t ntfs2utc (s64 ntfstime)
 {
   return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
 }
@@ -110,7 +114,7 @@ time_t ntfs2utc (s64 ntfstime)
  * list_entry
  * FIXME: Should we print errors as we go along? (AIA)
  */
-int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
+static int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type)
 {
@@ -260,7 +264,7 @@ free:
  * Return:  inode  Success, the pathname was valid
  *	    NULL   Error, the pathname was invalid, or some other error occurred
  */
-ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname)
+static ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname)
 {
 	u64 inum;
 	int len;
@@ -338,6 +342,38 @@ close:
 	return result;
 }
 
+static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
+{
+  ntfs_inode *inode;
+  s64 pos;
+  struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
+  ls->dir_list=NULL;
+  ls->current_file=NULL;
+
+  inode = utils_pathname_to_inode (ls->vol, NULL, dir_data->current_directory);
+  if (!inode) {
+    ecrit_rapport("ntfs_dir: utils_pathname_to_inode failed\n");
+    return NULL;
+  }
+
+  /*
+   * We now are at the final path component.  If it is a file just
+   * list it.  If it is a directory, list its contents.
+   */
+  pos = 0;
+  if (inode->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
+    if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)list_entry)<0)
+    {
+      ecrit_rapport("ntfs_readdir failed\n");
+    }
+  }
+  else
+    ecrit_rapport("ntfs_readdir BUG not MFT_RECORD_IS_DIRECTORY\n");
+  /* Finished with the inode; release it. */
+  ntfs_inode_close(inode);
+  return ls->dir_list;
+}
+
 /**
  * find_attribute - Find an attribute of the given type
  * @type:  An attribute type, e.g. AT_FILE_NAME
@@ -350,60 +386,164 @@ close:
  *       has been created without an inode, it won't overflow the buffer.
  *
  * Return:  Pointer  Success, an attribute was found
- *          NULL     Error, no matching attributes were found
+ *	    NULL     Error, no matching attributes were found
  */
-ATTR_RECORD * find_attribute (const ATTR_TYPES type, ntfs_attr_search_ctx *ctx)
+static ATTR_RECORD * find_attribute(const ATTR_TYPES type, ntfs_attr_search_ctx *ctx)
 {
-        if (!ctx) {
-                errno = EINVAL;
-                return NULL;
-        }
-                                                                                                                             
-        if (ntfs_attr_lookup(type, NULL, 0, 0, 0, NULL, 0, ctx) != 0) {
-                ecrit_rapport("find_attribute didn't find an attribute of type: 0x%02x.\n", type);
-                return NULL;    /* None / no more of that type */
-        }
-                                                                                                                             
-        ecrit_rapport("find_attribute found an attribute of type: 0x%02x.\n", type);
-        return ctx->attr;
+	if (!ctx) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	if (ntfs_attr_lookup(type, NULL, 0, 0, 0, NULL, 0, ctx) != 0) {
+		ecrit_rapport("find_attribute didn't find an attribute of type: 0x%02x.\n", type);
+		return NULL;	/* None / no more of that type */
+	}
+
+	ecrit_rapport("find_attribute found an attribute of type: 0x%02x.\n", type);
+	return ctx->attr;
 }
 
-t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
-t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
+/**
+ * find_first_attribute - Find the first attribute of a given type
+ * @type:  An attribute type, e.g. AT_FILE_NAME
+ * @mft:   A buffer containing a raw MFT record
+ *
+ * Search through a raw MFT record for an attribute of a given type.
+ * The return value is a pointer into the MFT record that was supplied.
+ *
+ * N.B.  This will return a pointer into @mft.  The pointer won't stray outside
+ *       the buffer, since we created the search context without an inode.
+ *
+ * Return:  Pointer  Success, an attribute was found
+ *	    NULL     Error, no matching attributes were found
+ */
+static ATTR_RECORD * find_first_attribute(const ATTR_TYPES type, MFT_RECORD *mft)
 {
-  ntfs_inode *ni;
-  s64 pos;
+	ntfs_attr_search_ctx *ctx;
+	ATTR_RECORD *rec;
+
+	if (!mft) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	ctx = ntfs_attr_get_search_ctx(NULL, mft);
+	if (!ctx) {
+		ecrit_rapport("Couldn't create a search context.\n");
+		return NULL;
+	}
+
+	rec = find_attribute(type, ctx);
+	ntfs_attr_put_search_ctx(ctx);
+	if (rec)
+		ecrit_rapport("find_first_attribute: found attr of type 0x%02x.\n", type);
+	else
+		ecrit_rapport("find_first_attribute: didn't find attr of type 0x%02x.\n", type);
+	return rec;
+}
+
+/**
+ * index_get_size - Find the INDX block size from the index root
+ * @inode:  Inode of the directory to be checked
+ *
+ * Find the size of a directory's INDX block from the INDEX_ROOT attribute.
+ *
+ * Return:  n  Success, the INDX blocks are n bytes in size
+ *	    0  Error, not a directory
+ */
+static int index_get_size(ntfs_inode *inode)
+{
+	ATTR_RECORD *attr90;
+	INDEX_ROOT *iroot;
+
+	attr90 = find_first_attribute(AT_INDEX_ROOT, inode->mrec);
+	if (!attr90)
+		return 0;	// not a directory
+
+	iroot = (INDEX_ROOT*)((u8*)attr90 + le16_to_cpu(attr90->value_offset));
+
+	return iroot->index_block_size;
+}
+
+static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_inode)
+{
+  ntfs_inode *inode;
   struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
-  ls->dir_list=NULL;
-  ls->current_file=NULL;
-
-  ni = utils_pathname_to_inode (ls->vol, NULL, dir_data->current_directory);
-  if (!ni) {
-    ecrit_rapport("ntfs_dir: utils_pathname_to_inode failed\n");
-    return NULL;
+  inode = utils_pathname_to_inode (ls->vol, NULL, dir_data->current_directory);
+  if (!inode) {
+    ecrit_rapport("ntfs_copy: utils_pathname_to_inode failed\n");
+    return 1;
   }
-
-  /*
-   * We now are at the final path component.  If it is a file just
-   * list it.  If it is a directory, list its contents.
-   */
-  pos = 0;
-  if (ni->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
-    if(ntfs_readdir(ni, &pos, ls, (ntfs_filldir_t)list_entry)<0)
-    {
-      ecrit_rapport("ntfs_readdir failed\n");
-    }
-  }
-  else
   {
-    ecrit_rapport("ntfs_readdir BUG not MFT_RECORD_IS_DIRECTORY\n");
-    ecrit_rapport("This file must be compiled with -fms-extensions\n");
+    const int bufsize = 4096;
+    char *buffer;
+    char *new_file;
+    ntfs_attr *attr;
+    FILE *f_out;
+    s64 bytes_read, written;
+    s64 offset;
+    u32 block_size;
+    buffer = MALLOC(bufsize);
+    if (!buffer)
+      return 2;
+    attr = ntfs_attr_open(inode, AT_DATA, NULL, 0);
+    if (!attr)
+    {
+      ecrit_rapport("Cannot find attribute type 0x%lx.\n", (long) AT_DATA);
+      free(buffer);
+      return 3;
+    }
+    if ((inode->mft_no < 2) && (attr->type == AT_DATA))
+      block_size = ls->vol->mft_record_size;
+    else if (attr->type == AT_INDEX_ALLOCATION)
+      block_size = index_get_size(inode);
+    else
+      block_size = 0;
+    new_file=MALLOC(strlen(dir_data->current_directory)+2);
+    new_file[0]='.';
+    strcpy(&new_file[1],dir_data->current_directory);
+    f_out=create_file(new_file);
+    if(!f_out)
+    {
+      ecrit_rapport("Can't create file %s: %s\n",new_file, strerror(errno));
+      FREE(new_file);
+      return 4;
+    }
+    FREE(new_file);
+    offset = 0;
+    for (;;)
+    {
+      if (block_size > 0) {
+	// These types have fixup
+	bytes_read = ntfs_attr_mst_pread(attr, offset, 1, block_size, buffer);
+	bytes_read *= block_size;
+      } else {
+	bytes_read = ntfs_attr_pread(attr, offset, bufsize, buffer);
+      }
+      //ntfs_log_info("read %lld bytes\n", bytes_read);
+      if (bytes_read == -1) {
+	ecrit_rapport("ERROR: Couldn't read file");
+	break;
+      }
+      if (!bytes_read)
+	break;
+
+      written = fwrite(buffer, 1, bytes_read, f_out);
+      if (written != bytes_read)
+      {
+	ecrit_rapport("ERROR: Couldn't output all data!");
+	break;
+      }
+      offset += bytes_read;
+    }
+    fclose(f_out);
+    ntfs_attr_close(attr);
+    FREE(buffer);
   }
-
   /* Finished with the inode; release it. */
-  ntfs_inode_close(ni);
-
-  return ls->dir_list;
+  ntfs_inode_close(inode);
+  return 0;
 }
 
 int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
@@ -433,6 +573,7 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
     screen_buffer_display(window,"",NULL);
     if(partition->boot_sector!=0)
       io_redir_del_redir(disk_car,partition->part_offset);
+    ntfs_device_free(dev);
     return 2;
   }
   if (vol->flags & VOLUME_IS_DIRTY) {
@@ -448,9 +589,11 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
     dir_data.debug=debug;
     dir_data.private_dir_data=&ls;
     dir_data.get_dir=ntfs_dir;
+    dir_data.copy_file=ntfs_copy;
     strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
     dir_partition(disk_car,partition,&dir_data,FILE_root,current_cmd);
-  }  
+  }
+  /* ntfs_umount() will invoke ntfs_device_free() for us. */
   ntfs_umount(vol, FALSE);
   if(partition->boot_sector!=0)
     io_redir_del_redir(disk_car,partition->part_offset);
@@ -469,7 +612,7 @@ const char*td_ntfs_version(void)
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
-#include "ntfs.h"
+#include "ntfs_dir.h"
 
 int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
 {
@@ -479,7 +622,10 @@ int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partit
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with ntfsprogs library");
-  screen_buffer_display(window,"",NULL);
+  if(*current_cmd!=NULL)
+    screen_buffer_to_log();
+  else
+    screen_buffer_display(window,"",NULL);
   return 0;
 }
 
