@@ -2,7 +2,7 @@
 
     File: parti386.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -103,7 +103,7 @@ static int write_part_i386(t_param_disk *disk_car, const t_list_part *list_part,
 static t_list_part *init_part_order_i386(const t_param_disk *disk_car, t_list_part *list_part);
 static int write_MBR_code_i386(t_param_disk *disk_car);
 static int write_MBR_code_i386_aux(unsigned char *buffer);
-static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_part, const int debug);
+static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
 static void set_prev_status_i386(const t_param_disk *disk_car, t_partition *partition);
 static void set_next_status_i386(const t_param_disk *disk_car, t_partition *partition);
 static int set_part_type_i386(t_partition *partition, unsigned int part_type);
@@ -187,6 +187,7 @@ static const struct systypes i386_sys_types[] = {
   {P_HFS,		"HFS"},
   {0xb7,		"BSDI"},
   {0xb8,		"BSDI swap"},
+  {0xbc,		"Acronis"},
   {0xbe,		"Solaris boot"},
   {P_SUN,		"Solaris"},
   {0xc1,		"secured FAT12"},
@@ -213,7 +214,8 @@ static const struct systypes i386_sys_types[] = {
 };
 
 t_arch_fnct arch_i386= {
-  .part_name="Partition table type : Intel",
+  .part_name="Intel",
+  .part_name_option="partition_i386",
   .msg_part_type="*=Primary bootable  P=Primary  L=Logical  E=Extended  D=Deleted",
   .read_part=read_part_i386,
   .write_part=write_part_i386,
@@ -334,20 +336,22 @@ static t_list_part *init_part_order_i386(const t_param_disk *disk_car, t_list_pa
 
 static t_list_part *read_part_i386(t_param_disk *disk_car, const int debug, const int saveheader)
 {
-  unsigned char buffer[disk_car->sector_size];
-  t_list_part *new_list_part=NULL;
   unsigned int i;
   int res=0;
   t_CHS geometry;
+  t_list_part *new_list_part=NULL;
+  unsigned char *buffer=MALLOC(disk_car->sector_size);
   aff_buffer(BUFFER_RESET,"Q");
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  if(disk_car->read(disk_car,disk_car->sector_size, buffer, (uint64_t)0))
   {
     wdoprintf(stdscr,msg_PART_RD_ERR);
+    FREE(buffer);
     return NULL;
   }
   if(get_geometry_from_i386mbr(buffer,debug,&geometry)!=0)
   {
     aff_buffer(BUFFER_ADD,msg_TBL_NMARK);
+    FREE(buffer);
     return NULL;
   }
   for(i=0;i<4;i++)
@@ -387,6 +391,7 @@ static t_list_part *read_part_i386(t_param_disk *disk_car, const int debug, cons
       new_list_part=get_ext_data_i386(disk_car,new_list_part,debug,saveheader);
   }
   get_geometry_from_list_part(disk_car, new_list_part, debug);
+  FREE(buffer);
   return new_list_part;
 }
 
@@ -614,8 +619,8 @@ int write_part_i386(t_param_disk *disk_car, const t_list_part *list_part, const
 static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug)
 {
   const t_list_part *element;
-  unsigned char buffer[disk_car->sector_size];
-  unsigned char buffer_org[disk_car->sector_size];
+  unsigned char *buffer=MALLOC(disk_car->sector_size);
+  unsigned char *buffer_org=MALLOC(disk_car->sector_size);
   if(debug>0)
   {
     ecrit_rapport("\nwrite_mbr_i386: starting...\n");
@@ -623,9 +628,9 @@ static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part,
   if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer_org, (uint64_t)0))
   {
     wdoprintf(stdscr,msg_PART_RD_ERR);
-    memset((void *)&buffer_org,0,DEFAULT_SECTOR_SIZE);
+    memset(buffer_org,0,DEFAULT_SECTOR_SIZE);
   }
-  memset((void *)&buffer,0,DEFAULT_SECTOR_SIZE);
+  memset(buffer,0,DEFAULT_SECTOR_SIZE);
   if((buffer_org[0x1FE]==0x55) && (buffer_org[0x1FF]==0xAA))
   {
     memcpy(buffer,buffer_org,TAB_PART);
@@ -670,9 +675,13 @@ static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part,
   {
     if(disk_car->write(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
     {
+      FREE(buffer_org);
+      FREE(buffer);
       return 1;
     }
   }
+  FREE(buffer_org);
+  FREE(buffer);
   return 0;
 }
 
@@ -879,11 +888,11 @@ static int write_MBR_code_i386_aux(unsigned char *buffer)
     0xfc, 0x31, 0xc0, 0x8e, 0xd0, 0x31, 0xe4, 0x8e, 0xd8, 0x8e, 0xc0, 0xbe, 0x00, 0x7c, 0xbf, 0x00,
     0x06, 0xb9, 0x00, 0x01, 0xf3, 0xa5, 0xbe, 0xee, 0x07, 0xb0, 0x08, 0xea, 0x20, 0x06, 0x00, 0x00,
     0x80, 0x3e, 0xb3, 0x07, 0xff, 0x75, 0x04, 0x88, 0x16, 0xb3, 0x07, 0x80, 0x3c, 0x00, 0x74, 0x04,
-    0x08, 0x06, 0xaf, 0x07, 0x83, 0xee, 0x10, 0xd0, 0xe8, 0x73, 0xf0, 0xcd, 0x1a, 0x89, 0x16, 0x00,
-    0x08, 0xe8, 0x2d, 0x01, 0x81, 0x3e, 0xb1, 0x07, 0xff, 0xff, 0x74, 0x46, 0xf6, 0x06, 0xb0, 0x07,
-    0x80, 0x74, 0x06, 0xb4, 0x01, 0xcd, 0x16, 0x75, 0x39, 0xf6, 0x06, 0xb0, 0x07, 0x40, 0x74, 0x07,
-    0xf6, 0x06, 0x17, 0x04, 0x0f, 0x75, 0x2b, 0x31, 0xc0, 0xcd, 0x1a, 0x2b, 0x16, 0x00, 0x08, 0x2b,
-    0x16, 0xb1, 0x07, 0x72, 0xd7, 0xa0, 0xb0, 0x07, 0x24, 0x07, 0x3c, 0x07, 0x75, 0x0b, 0xbe, 0xbe,
+    0x08, 0x06, 0xaf, 0x07, 0x83, 0xee, 0x10, 0xd0, 0xe8, 0x73, 0xf0, 0x90, 0x90, 0x90, 0x90, 0x90,
+    0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+    0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+    0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90,
+    0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0xbe, 0xbe,
     0x07, 0xb0, 0x00, 0xb9, 0x04, 0x00, 0x80, 0x3c, 0x00, 0x75, 0x6e, 0xfe, 0xc0, 0x83, 0xc6, 0x10,
     0xe2, 0xf4, 0x31, 0xdb, 0xb4, 0x0e, 0xbe, 0x9d, 0x07, 0x8a, 0x0e, 0xaf, 0x07, 0xac, 0xd0, 0xe9,
     0x73, 0x02, 0xcd, 0x10, 0x08, 0xc9, 0x75, 0xf5, 0xb0, 0x3a, 0xcd, 0x10, 0x31, 0xc0, 0xcd, 0x16,
@@ -1128,11 +1137,21 @@ static int test_structure_i386(t_list_part *list_part)
     if(element->part->status!=STATUS_DELETED)
       new_list_part=insert_new_partition(new_list_part,element->part);
   }
-  /* Test overlapping */
+  /* Test overlapping
+     Must be space between a primary/logical partition and a logical partition for an extended
+  */
   for(element=new_list_part;element!=NULL;element=element->next)
   {
-    if(((element->prev!=NULL) && (element->part->part_offset<=element->prev->part->part_offset+element->prev->part->part_size-1)) ||
-	((element->next!=NULL) && (element->part->part_offset+element->part->part_size-1>=element->next->part->part_offset)))
+    if(((element->prev!=NULL) &&
+	  (element->prev->part->part_offset+element->prev->part->part_size-1 >= element->part->part_offset)) ||
+	(element->prev!=NULL &&
+	 (element->prev->part->status==STATUS_PRIM ||
+	  element->prev->part->status==STATUS_PRIM_BOOT ||
+	  element->prev->part->status==STATUS_LOG) &&
+	 element->part->status==STATUS_LOG &&
+	 element->prev->part->part_offset+element->prev->part->part_size-1+1 >= element->part->part_offset) ||
+	((element->next!=NULL) &&
+	 (element->part->part_offset+element->part->part_size-1>=element->next->part->part_offset)))
     {
       res=1;
     }
@@ -1150,7 +1169,7 @@ static int is_extended(const unsigned int part_type)
   return (part_type==(const unsigned char)P_EXTENDX || part_type==(const unsigned char)P_EXTENDED || part_type==(const unsigned char)P_LINUXEXTENDX);
 }
 
-static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_part, const int debug)
+static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd)
 {
   t_CHS start,end;
   t_partition *new_partition=partition_new();
@@ -1216,7 +1235,7 @@ static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_
 	  break;
 	case 'T':
 	case 't':
-	  change_part_type(disk_car,new_partition);
+	  change_part_type(disk_car,new_partition,current_cmd);
 	  position=7;
 	  break;
 	case key_ESC:
