@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -46,6 +46,7 @@
 #include "cramfs.h"
 #include "ext2.h"
 #include "fat.h"
+#include "fatx.h"
 #include "hfs.h"
 #include "hfsp.h"
 #include "jfs_superblock.h"
@@ -65,7 +66,7 @@ static int check_part_none(t_param_disk *disk_car, const int debug,t_partition *
 static int get_geometry_from_nonembr(const unsigned char *buffer, const int debug, t_CHS *geometry);
 static t_list_part *read_part_none(t_param_disk *disk_car, const int debug, const int saveheader);
 static t_list_part *init_part_order_none(const t_param_disk *disk_car, t_list_part *list_part);
-static t_list_part *add_partition_none(t_param_disk *disk_car,t_list_part *list_part, const int debug);
+static t_list_part *add_partition_none(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
 static void set_next_status_none(const t_param_disk *disk_car, t_partition *partition);
 static int test_structure_none(t_list_part *list_part);
 static int set_part_type_none(t_partition *partition, unsigned int part_type);
@@ -101,6 +102,7 @@ static const struct systypes none_sys_types[] = {
   {UP_RFS,	"ReiserFS"},
   {UP_RFS2,	"ReiserFS 2"},
   {UP_RFS3,	"ReiserFS 3"},
+  {UP_RFS4,	"ReiserFS 4"},
   {UP_SUN,	"Sun"},
   {UP_SYSV4,	"SysV 4"},
   {UP_UFS,	"UFS"},
@@ -115,7 +117,8 @@ static const struct systypes none_sys_types[] = {
 
 t_arch_fnct arch_none=
 {
-  .part_name="Partition table type : None",
+  .part_name="None",
+  .part_name_option="partition_none",
   .msg_part_type=NULL,
   .read_part=read_part_none,
   .write_part=NULL,
@@ -145,6 +148,19 @@ int get_geometry_from_nonembr(const unsigned char *buffer, const int debug, t_CH
   geometry->cylinder=0;
   geometry->head=0;
   geometry->sector=0;
+  {
+    /* Ugly hack to get geometry from FAT and NTFS */
+    const struct fat_boot_sector *fat_header=(const struct fat_boot_sector *)buffer;
+    if(le16(fat_header->marker)==0xAA55)
+    {
+      if(le16(fat_header->secs_track)>0 && le16(fat_header->secs_track)<=63 &&
+	  le16(fat_header->heads)>0 && le16(fat_header->heads)<=255)
+      {
+	geometry->sector=le16(fat_header->secs_track);
+	geometry->head=le16(fat_header->heads)-1;
+      }
+    }
+  }
   return 0;
 }
 
@@ -194,7 +210,7 @@ static t_list_part *init_part_order_none(const t_param_disk *disk_car, t_list_pa
   return list_part;
 }
 
-static t_list_part *add_partition_none(t_param_disk *disk_car,t_list_part *list_part, const int debug)
+static t_list_part *add_partition_none(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd)
 {
   return list_part;
 }
@@ -250,6 +266,9 @@ static int check_part_none(t_param_disk *disk_car,const int debug,t_partition *p
     case UP_FAT32:
       ret=check_FAT(disk_car,partition,debug);
       break;
+    case UP_FATX:
+      ret=check_FATX(disk_car,partition,debug);
+      break;
     case UP_FREEBSD:
       ret=check_BSD(disk_car,partition,debug,BSD_MAXPARTITIONS);
       break;
@@ -297,6 +316,7 @@ static int check_part_none(t_param_disk *disk_car,const int debug,t_partition *p
     case UP_RFS:
     case UP_RFS2:
     case UP_RFS3:
+    case UP_RFS4:
 	ret=check_rfs(disk_car,partition,debug);
 	break;
     case UP_SUN:
