@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -61,7 +61,7 @@ static int get_geometry_from_sunmbr(const unsigned char *buffer, const int debug
 static t_list_part *read_part_sun(t_param_disk *disk_car, const int debug, const int saveheader);
 static int write_part_sun(t_param_disk *disk_car, const t_list_part *list_part, const int ro , const int debug, const int align);
 static t_list_part *init_part_order_sun(const t_param_disk *disk_car, t_list_part *list_part);
-static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug);
+static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
 static void set_next_status_sun(const t_param_disk *disk_car, t_partition *partition);
 static int test_structure_sun(t_list_part *list_part);
 static int set_part_type_sun(t_partition *partition, unsigned int part_type_sun);
@@ -92,7 +92,8 @@ static const struct systypes sun_sys_types[] = {
 
 t_arch_fnct arch_sun=
 {
-  .part_name="Partition table type : Sun",
+  .part_name="Sun",
+  .part_name_option="partition_sun",
   .msg_part_type="                P=Primary  D=Deleted",
   .read_part=read_part_sun,
   .write_part=write_part_sun,
@@ -210,7 +211,7 @@ static t_list_part *init_part_order_sun(const t_param_disk *disk_car, t_list_par
   return insert_new_partition(list_part, new_partition);
 }
 
-static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug)
+static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd)
 {
   t_CHS start,end;
   t_partition *new_partition=partition_new();
@@ -256,7 +257,7 @@ static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_p
 	  break;
 	case 'T':
 	case 't':
-	  change_part_type(disk_car,new_partition);
+	  change_part_type(disk_car,new_partition,current_cmd);
 	  position=3;
 	  break;
 	case key_ESC:
