@@ -2,7 +2,7 @@
 
     File: photorec.h
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -30,6 +30,7 @@ typedef struct file_stat_struct t_file_stat;
 struct file_stat_struct
 {
   unsigned int enable;
+  unsigned int ifnothingfound;
   unsigned int not_recovered;
   unsigned int recovered;
   const t_file_hint *file_hint;
@@ -50,16 +51,27 @@ struct file_hint_struct
 {
   const char *extension;
   const char *description;
-  uint64_t min_filesize;
+  const uint64_t min_header_distance;
+  const uint64_t min_filesize;
   /* don't try head_check if min_header_distance >0 and previous_header_distance <= min_header_distance */
   /* needed by tar header */
-  uint64_t min_header_distance;
-  uint64_t max_filesize;
-  int stream;
-  int recover;
-  int (*header_check)(const unsigned char *buffer, const unsigned int buffer_size);
+  const uint64_t max_filesize;
+  const int recover;
+  const char *(*header_check)(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, t_file_recovery *file_recovery);
   int (*data_check)(const unsigned char*buffer, const unsigned int buffer_size, t_file_recovery *file_recovery);
   /* 0: bad, 1: EOF not found, 2: EOF*/
   void (*file_check)(t_file_recovery *file_recovery);
 };
 
+typedef struct struct_alloc_data t_alloc_data;
+
+struct struct_alloc_data
+{
+  uint64_t start;
+  uint64_t end;
+  t_alloc_data *prev;
+  t_alloc_data *next;
+  t_file_stat *file_stat;
+};
+
+
