@@ -2,7 +2,7 @@
 
     File: rfs.h
 
-    Copyright (C) 2005  Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2005-2006  Christophe GRENIER <grenier@cgsecurity.org>
     Taken from ReiserFS v0.91. Reiserfs Copyright 1996, 1997, 1998 Hans Reiser
   
     This software is free software; you can redistribute it and/or modify
@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -69,6 +69,38 @@ struct reiserfs_super_block
 
 #define SB_SIZE (sizeof(struct reiserfs_super_block))
 
+#define REISERFS4_SUPER_MAGIC		"ReIsEr4"
+#define MAGIC_SIZE 16
+typedef struct reiser4_master_sb reiser4_master_sb;
+struct reiser4_master_sb {
+  char magic[16];         /* "ReIsEr4" */
+  uint16_t disk_plugin_id;     /* id of disk layout plugin */
+  uint16_t blocksize;
+  char uuid[16];          /* unique id */
+  char label[16];         /* filesystem label */
+  uint64_t diskmap;            /* location of the diskmap. 0 if not present */
+} __attribute__ ((__packed__));
+
+struct format40_super {
+  uint64_t sb_block_count;
+  uint64_t sb_free_blocks;
+  uint64_t sb_root_block;
+
+  /* These 2 fields are for oid data. */
+  uint64_t sb_oid[2];
+
+  uint64_t sb_flushes;
+
+  uint32_t sb_mkfs_id;
+  char sb_magic[MAGIC_SIZE];
+
+  uint16_t sb_tree_height;
+  uint16_t sb_policy;
+  uint64_t sb_flags;
+
+  char sb_unused[432];
+} __attribute__((packed));
+
 int check_rfs(t_param_disk *disk_car,t_partition *partition,const int debug);
 int recover_rfs(t_param_disk *disk_car, const struct reiserfs_super_block *sb,t_partition *partition,const int debug, const int dump_ind);
 
