@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -489,6 +489,7 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *part
     dir_data.debug=debug;
     dir_data.private_dir_data=&ls;
     dir_data.get_dir=reiser_dir;
+    dir_data.copy_file=NULL;
     strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
     dir_partition(disk_car,partition,&dir_data,2, current_cmd);
   }
@@ -510,7 +511,7 @@ const char*td_reiserfs_version(void)
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
-#include "rfs.h"
+#include "rfs_dir.h"
 
 int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
 {
@@ -519,7 +520,10 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *part
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with progsreiserfs library");
-  screen_buffer_display(window,"",NULL);
+  if(*current_cmd!=NULL)
+    screen_buffer_to_log();
+  else
+    screen_buffer_display(window,"",NULL);
   return 0;
 }
 
