@@ -2,7 +2,7 @@
 
     File: testdisk.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
   
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write the Free Software Foundation, Inc., 59
+    Temple Place - Suite 330, Boston MA 02111-1307, USA.
 
  */
 
@@ -28,7 +28,7 @@
 #endif
 #include <stdarg.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>	/* geteuid */
+#include <unistd.h>	/* dup2 */
 #endif
 #ifdef HAVE_STRING_H
 #include <string.h>
@@ -58,6 +58,7 @@
 #include "ext2_dir.h"
 #include "rfs_dir.h"
 #include "ntfs_dir.h"
+#include "hdcache.h"
 extern const t_arch_fnct arch_i386;
 extern const t_arch_fnct arch_mac;
 extern const t_arch_fnct arch_sun;
@@ -80,7 +81,6 @@ int ecrit_rapport(const char *_format, ...)
   if(f_rapport!=NULL)
   {
     va_list ap;
-    fflush(stderr);
     va_start(ap,_format);
     res=vfprintf(f_rapport,_format,ap);
     va_end(ap);
@@ -88,6 +88,7 @@ int ecrit_rapport(const char *_format, ...)
     {
       f_status=1;
     }
+    fflush(stderr);
     if(fflush(f_rapport))
     {
       f_status=1;
@@ -152,11 +153,10 @@ int main( int argc, char **argv )
   int i;
   int help=0, create_log=0, debug=0, dump_ind=0;
   int do_list=0;
-  int test_recovery=0;
   int write_used;
   int saveheader=0;
   int create_backup=0;
-  int use_direct_io=0;
+  int testdisk_mode=TESTDISK_O_RDWR|TESTDISK_O_READAHEAD_8K;
   t_list_disk *list_disk=NULL;
   t_list_disk *element_disk;
   const char *cmd_device=NULL;
@@ -184,7 +184,7 @@ int main( int argc, char **argv )
     return -1;
   }
 #endif
-#if defined(__CYGWIN__) || defined(__MINGW32__)
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__APPLE__)
   /* Average Windows user doesn't know how to specify a parameter */
   if(argc==1)
   {
@@ -194,13 +194,7 @@ int main( int argc, char **argv )
 #endif
   for(i=1;i<argc;i++)
   {
-    if((strcmp(argv[i],"/test_recovery")==0) ||(strcmp(argv[i],"-test_recovery")==0))
-    {
-      test_recovery=1;
-      do_list=1;
-      create_log=1;
-    }
-    else if((strcmp(argv[i],"/dump")==0) || (strcmp(argv[i],"-dump")==0))
+    if((strcmp(argv[i],"/dump")==0) || (strcmp(argv[i],"-dump")==0))
       dump_ind=1;
     else if((strcmp(argv[i],"/log")==0) ||(strcmp(argv[i],"-log")==0))
     {
@@ -218,7 +212,7 @@ int main( int argc, char **argv )
     else if((strcmp(argv[i],"/backup")==0) || (strcmp(argv[i],"-backup")==0))
       create_backup=1;
     else if((strcmp(argv[i],"/direct")==0) || (strcmp(argv[i],"-direct")==0))
-      use_direct_io=1;
+      testdisk_mode|=TESTDISK_O_DIRECT;
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
@@ -228,17 +222,29 @@ int main( int argc, char **argv )
 	help=1;
       else
       {
+	t_param_disk *disk_car;
 	cmd_device=argv[++i];
 	cmd_run=argv[++i];
+	disk_car=file_test_availability(cmd_device,debug,arch,testdisk_mode);
+	if(disk_car==NULL)
+	{
+	  printf("\nUnable to open file or device %s\n",cmd_device);
+	  help=1;
+	}
+	else
+	  list_disk=insert_new_disk(list_disk,disk_car);
       }
     }
     else
     {
-      list_disk=insert_new_disk(list_disk,file_test_availability(argv[i],debug,arch,use_direct_io));
-      if(list_disk==NULL)
+      t_param_disk *disk_car=file_test_availability(argv[i],debug,arch,testdisk_mode);
+      if(disk_car==NULL)
       {
+	printf("\nUnable to open file or device %s\n",argv[i]);
 	help=1;
       }
+      else
+	list_disk=insert_new_disk(list_disk,disk_car);
     }
   }
   if(cmd_device!=NULL)
@@ -286,6 +292,8 @@ int main( int argc, char **argv )
     ecrit_rapport("Solaris version");
 #elif defined(__CYGWIN__) || defined(__MINGW32__)
     ecrit_rapport("Windows version");
+#elif defined(__APPLE__)
+    ecrit_rapport("Apple version");
 #else
     ecrit_rapport("Undefined OS");
 #endif
@@ -305,33 +313,19 @@ int main( int argc, char **argv )
   }
 #endif
   aff_buffer(BUFFER_RESET,"Q");
-  list_disk=hd_parse(list_disk,debug,arch,use_direct_io);
+  /* Scan for avaible device only if no device or image has been supplied in parameter */
   if(list_disk==NULL)
-  {
-    printf("No harddisk found\n");
-#if defined(__CYGWIN__) || defined(__MINGW32__)
-    printf("You need to be administrator to use TestDisk\n");
-    printf("Under Win9x, use the DOS version instead\n");
-    ecrit_rapport("You need to be administrator to use TestDisk\n");
-#else
-#ifndef DJGPP
-#ifdef HAVE_GETEUID
-    if(geteuid()!=0)
-    {
-      printf("You need to be root to use TestDisk\n");
-      ecrit_rapport("You need to be root to use TestDisk\n");
-    }
-#endif
-#endif
-#endif
-  }
+    list_disk=hd_parse(list_disk,debug,arch,testdisk_mode);
+  /* Activate the cache */
+  for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
+    element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
 #ifdef DJGPP
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
   {
     printf("%s\n",element_disk->disk->description(element_disk->disk));
   }
 #endif
-  hd_parse_bis(list_disk,0,arch,debug);
+  hd_update_all_geometry(list_disk,0,debug);
   /* save disk parameters to rapport */
   ecrit_rapport("Hard disk list\n");
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
@@ -345,24 +339,12 @@ int main( int argc, char **argv )
   {
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
-      interface_list(element_disk->disk,debug,test_recovery,saveheader,create_backup);
-      if(test_recovery!=0)
-      {
-	t_list_part *list_part=search_part(element_disk->disk,0,debug,0,0,0);
-	if(debug>0)
-	{
-	  t_list_part *element;
-	  ecrit_rapport("search_part() results\n");
-	  for(element=list_part;element!=NULL;element=element->next)
-	    aff_part_rapport(element_disk->disk,element->part);
-	}
-	delete_list_part(list_part);
-      }
+      interface_list(element_disk->disk,debug,0,saveheader,create_backup);
     }
   }
   else
   {
-    do_curses_testdisk(debug,dump_ind,list_disk,saveheader,&arch,cmd_device,cmd_run);
+    do_curses_testdisk(debug,dump_ind,list_disk,saveheader,argv[0],cmd_device,cmd_run);
     cmd_device=NULL;
     cmd_run=NULL;
   }
