@@ -268,9 +268,6 @@
 #define UFS2_MAXVOLLEN 32
 #define UFS_MAXCSBUFS 31
 #define UFS_LINK_MAX 32000
-/*
-#define	UFS2_NOCSPTRS	((128 / sizeof(void *)) - 4)
-*/
 #define	UFS2_NOCSPTRS	28
 
 /*
@@ -284,14 +281,14 @@
 struct ufs_timeval {
 	uint32_t	tv_sec;
 	uint32_t	tv_usec;
-};
+} __attribute__ ((__packed__));
 
 struct ufs_csum {
 	uint32_t	cs_ndir;	/* number of directories */
 	uint32_t	cs_nbfree;	/* number of free blocks */
 	uint32_t	cs_nifree;	/* number of free inodes */
 	uint32_t	cs_nffree;	/* number of free frags */
-};
+} __attribute__ ((__packed__));
 struct ufs2_csum_total {
 	uint64_t	cs_ndir;	/* number of directories */
 	uint64_t	cs_nbfree;	/* number of free blocks */
@@ -299,11 +296,35 @@ struct ufs2_csum_total {
 	uint64_t	cs_nffree;	/* number of free frags */
 	uint64_t   cs_numclusters;	/* number of free clusters */
 	uint64_t   cs_spare[3];	/* future expansion */
-};
+} __attribute__ ((__packed__));
 
 /*
  * This is the actual superblock, as it is laid out on the disk.
  */
+struct fs_u11_u2_st {
+	int8_t  fs_fsmnt[UFS2_MAXMNTLEN];	/* name mounted on */
+	uint8_t   fs_volname[UFS2_MAXVOLLEN]; /* volume name */
+	uint64_t  fs_swuid;		/* system-wide uid */
+	uint32_t  fs_pad;	/* due to alignment of fs_swuid */
+	uint32_t   fs_cgrotor;     /* last cg searched */
+	uint32_t   fs_ocsp[UFS2_NOCSPTRS]; /*list of fs_cs info buffers */
+	uint32_t   fs_contigdirs;/*# of contiguously allocated dirs */
+	uint32_t   fs_csp;	/* cg summary info buffer for fs_cs */
+	uint32_t   fs_maxcluster;
+	uint32_t   fs_active;/* used by snapshots to track fs */
+	uint32_t   fs_old_cpc;	/* cyl per cycle in postbl */
+	uint32_t   fs_maxbsize;/*maximum blocking factor permitted */
+	uint64_t   fs_sparecon64[17];/*old rotation block list head */
+	uint64_t   fs_sblockloc; /* byte offset of standard superblock */
+	struct  ufs2_csum_total fs_cstotal;/*cylinder summary information*/
+	struct  ufs_timeval    fs_time;		/* last time written */
+	uint64_t    fs_size;		/* number of blocks in fs */
+	uint64_t    fs_dsize;	/* number of data blocks in fs */
+	uint64_t   fs_csaddr;	/* blk addr of cyl grp summary area */
+	uint64_t    fs_pendingblocks;/* blocks in process of being freed */
+	uint32_t    fs_pendinginodes;/*inodes in process of being freed */
+} __attribute__ ((__packed__));
+
 struct ufs_super_block {
 	uint32_t	fs_link;	/* UNUSED */
 	uint32_t	fs_rlink;	/* UNUSED */
@@ -388,31 +409,9 @@ struct ufs_super_block {
 			uint32_t	fs_maxcluster;
 			uint32_t	fs_cpc;		/* cyl per cycle in postbl */
 			uint16_t	fs_opostbl[16][8]; /* old rotation block list head */
-		} fs_u1;
-		struct {
-			int8_t  fs_fsmnt[UFS2_MAXMNTLEN];	/* name mounted on */
-			uint8_t   fs_volname[UFS2_MAXVOLLEN]; /* volume name */
-			uint64_t  fs_swuid;		/* system-wide uid */
-			uint32_t  fs_pad;	/* due to alignment of fs_swuid */
-			uint32_t   fs_cgrotor;     /* last cg searched */
-			uint32_t   fs_ocsp[UFS2_NOCSPTRS]; /*list of fs_cs info buffers */
-			uint32_t   fs_contigdirs;/*# of contiguously allocated dirs */
-			uint32_t   fs_csp;	/* cg summary info buffer for fs_cs */
-			uint32_t   fs_maxcluster;
-			uint32_t   fs_active;/* used by snapshots to track fs */
-			uint32_t   fs_old_cpc;	/* cyl per cycle in postbl */
-			uint32_t   fs_maxbsize;/*maximum blocking factor permitted */
-			uint64_t   fs_sparecon64[17];/*old rotation block list head */
-			uint64_t   fs_sblockloc; /* byte offset of standard superblock */
-			struct  ufs2_csum_total fs_cstotal;/*cylinder summary information*/
-			struct  ufs_timeval    fs_time;		/* last time written */
-			uint64_t    fs_size;		/* number of blocks in fs */
-			uint64_t    fs_dsize;	/* number of data blocks in fs */
-			uint64_t   fs_csaddr;	/* blk addr of cyl grp summary area */
-			uint64_t    fs_pendingblocks;/* blocks in process of being freed */
-			uint32_t    fs_pendinginodes;/*inodes in process of being freed */
-		} fs_u2;
-	}  fs_u11;
+		} fs_u1 __attribute__ ((__packed__));
+		struct fs_u11_u2_st fs_u2;
+	}  fs_u11 __attribute__ ((__packed__));
 	union {
 		struct {
 			uint32_t	fs_sparecon[53];/* reserved for future constants */
@@ -447,7 +446,7 @@ struct ufs_super_block {
 	uint32_t	fs_rotbloff;		/* (uint8_t) blocks for each rotation */
 	uint32_t	fs_magic;		/* magic number */
 	uint8_t	fs_space[1];		/* list of blocks for each rotation */
-};
+} __attribute__ ((__packed__));
 
 /*
  * Preference for optimization.
