@@ -2,7 +2,7 @@
 
     File: xfs.c
 
-    Copyright (C) 2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ static int test_xfs(const t_param_disk *disk_car, const struct xfs_sb *sb,t_part
   }
   else
     return 1;
-  if(debug!=0)
+  if(debug>0)
     ecrit_rapport("\nXFS Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   return 0;
 }
@@ -81,7 +81,7 @@ int recover_xfs(t_param_disk *disk_car, const struct xfs_sb *sb,t_partition *par
 {
   if(test_xfs(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
-  if(debug!=0 || dump_ind!=0)
+  if(debug>0 || dump_ind!=0)
   {
     ecrit_rapport("\nrecover_xfs\n");
     if(dump_ind!=0)
@@ -91,13 +91,16 @@ int recover_xfs(t_param_disk *disk_car, const struct xfs_sb *sb,t_partition *par
   }
 
   partition->part_size = (uint64_t)be64(sb->sb_dblocks) * be32(sb->sb_blocksize);
-  partition->part_type = (unsigned char)P_LINUX;
+  partition->part_type_i386=P_LINUX;
+  partition->part_type_mac=PMAC_LINUX;
+  partition->part_type_sun=PSUN_LINUX;
   set_xfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
 
 static int set_xfs_info(const t_param_disk *disk_car,const struct xfs_sb *sb,t_partition *partition, const int debug, const int dump_ind)
 {
+  partition->name[0]='\0';
   partition->info[0]='\0';
   switch(partition->upart_type)
   {
@@ -109,8 +112,9 @@ static int set_xfs_info(const t_param_disk *disk_car,const struct xfs_sb *sb,t_p
       break;
     case UP_XFS3:
       strncpy(partition->info,"XFS 6.2 - new inode version",sizeof(partition->info));
+      break;
     case UP_XFS4:
-	strncpy(partition->info,"XFS 6.2+ - bitmap version",sizeof(partition->info));
+      strncpy(partition->info,"XFS 6.2+ - bitmap version",sizeof(partition->info));
       break;
     default:
       return 1;
